define(["require", "exports", "./baseConfiguration"], function (require, exports, baseConfiguration_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class DamageCalculationConfiguration extends baseConfiguration_1.BaseConfiguration {
        constructor() {
            super(...arguments);
            this.entries = new Map();
        }
        get configName() {
            return "damageCalculation";
        }
        initWithJson(json) {
            let len = Object.keys(json).length;
            for (let i = 0; i < len; i++) {
                let b = json[i];
                let key = (b.atkType << 15) | b.defType;
                this.entries.set(key, b);
            }
        }
        getRatio(atkType, defType) {
            if (atkType < 1 || defType < 1) {
                return 1;
            }
            let lvS = (atkType << 15) | defType;
            if (this.entries.has(lvS)) {
                return this.entries.get(lvS).ratio;
            }
            console.warn(`get ratio [${atkType}] - [${defType}] failed`);
            return null;
        }
        getRatioByDefType(defType) {
            let arr = new Array();
            for (let entry of this.entries.values()) {
                if (entry.defType == defType) {
                    arr.push(entry);
                }
            }
            return arr;
        }
        get size() {
            return this.entries.size;
        }
    }
    exports.DamageCalculationConfiguration = DamageCalculationConfiguration;
});
//# sourceMappingURL=damageCalculationConfiguration.js.map