define(["require", "exports", "./baseConfiguration"], function (require, exports, baseConfiguration_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class MonsterEmitterConfiguration extends baseConfiguration_1.BaseConfiguration {
        constructor() {
            super(...arguments);
            this.entries = new Map();
            this.maxLevel = 1;
        }
        get configName() {
            return "monsterEmitter";
        }
        initWithJson(json) {
            let len = Object.keys(json).length;
            for (let i = 0; i < len; i++) {
                let b = json[i];
                this.entries.set(b.id, b);
                if (b.level > this.maxLevel) {
                    this.maxLevel = b.level;
                }
            }
        }
        getConfig(id) {
            if (this.entries.has(id)) {
                return this.entries.get(id);
            }
            console.warn(`get monster emitter [${id}] failed`);
            return null;
        }
        isMaxConfigId(level, configId) {
            for (let entry of this.entries.values()) {
                if (entry.level > level) {
                    continue;
                }
                if (entry.id > configId) {
                    return false;
                }
            }
            return true;
        }
        isMaxLevel(level) {
            return level >= this.maxLevel;
        }
        get size() {
            return this.entries.size;
        }
    }
    exports.MonsterEmitterConfiguration = MonsterEmitterConfiguration;
});
//# sourceMappingURL=monsterEmitterConfiguration.js.map