var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
define(["require", "exports", "./baseLogic", "../models/npcModel", "../models/modelStore", "../utils/configUtil", "../configs/npcDefaultSkillConfiguration", "../models/skillModel", "../configs/npcConfiguration", "../models/playerModel", "../models/achievementModel", "../configs/achievementConfiguration", "../models/monsterEmitterModel", "../configs/gameConfiguration", "../models/battlefieldModel"], function (require, exports, baseLogic_1, npcModel_1, modelStore_1, configUtil_1, npcDefaultSkillConfiguration_1, skillModel_1, npcConfiguration_1, playerModel_1, achievementModel_1, achievementConfiguration_1, monsterEmitterModel_1, gameConfiguration_1, battlefieldModel_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class PlayerLogic extends baseLogic_1.BaseLogic {
        constructor() {
            super(...arguments);
            this.syncRemote = false;
            this.lastSyncRemote = 0;
            this.recoveringAccount = false;
        }
        notifyEventS(event, eventData) {
            super.notifyEventS(event, eventData);
        }
        onEventS(event, caller, listener, once = true) {
            return super.onEventS(event, caller, listener, once);
        }
        initPlayerAsync() {
            return __awaiter(this, void 0, void 0, function* () {
                const store = this.serviceProvider.getService(modelStore_1.ModelStore);
                const gc = this.configUtil.getConfig(gameConfiguration_1.GameConfiguration);
                const uuid = yield this.platform.getUUIDAsync();
                const key = playerModel_1.PlayerModel.genKey(uuid);
                let db = yield store.loadDbAsync(key);
                let player;
                if (db == null) {
                    db = {
                        id: uuid,
                        gold: gc.getNum("initGold"),
                        magicEssence: 0,
                        gauge: gc.getNum("initGauge") * 1000,
                        lastGetReward: 0,
                        passGuide: false,
                        hasLinkAccount: false
                    };
                    player = new playerModel_1.PlayerModel(db);
                    yield store.saveDbAsync(player);
                }
                else {
                    player = new playerModel_1.PlayerModel(db);
                }
                store.saveMe(player);
                store.save(player);
            });
        }
        initAchievementAsync() {
            return __awaiter(this, void 0, void 0, function* () {
                const store = this.serviceProvider.getService(modelStore_1.ModelStore);
                const configs = this.configUtil.getConfig(achievementConfiguration_1.AchievementConfiguration);
                const player = store.loadMe();
                const dbkey = "achievement";
                let db = yield store.loadDbAsync(dbkey);
                let achievements = [];
                let existTypes = new Set();
                if (db == null) {
                    db = [];
                }
                else {
                    for (let dbm of db) {
                        existTypes.add(dbm.type);
                    }
                }
                const orderedList = configs.getOrderedList();
                for (let config of orderedList) {
                    if (existTypes.has(config.type)) {
                        continue;
                    }
                    existTypes.add(config.type);
                    const dbmodel = { type: config.type, progress: 0, configId: config.id };
                    db.push(dbmodel);
                }
                for (let dbmodel of db) {
                    const ach = new achievementModel_1.AchievementModel(dbmodel);
                    const config = configs.get(dbmodel.configId);
                    ach.event = config.event;
                    achievements.push(ach);
                }
                for (let ach of achievements) {
                    const key = ach.genKey();
                    store.save(ach, key);
                    store.saveKeyByType(player.id, dbkey, key);
                }
            });
        }
        initNpcAsync() {
            return __awaiter(this, void 0, void 0, function* () {
                const store = this.serviceProvider.getService(modelStore_1.ModelStore);
                const player = store.loadMe();
                const dbkey = "npc";
                let db = yield store.loadDbAsync(dbkey);
                let npcs = [];
                if (db != null) {
                    for (let dbmodel of db) {
                        const ach = new npcModel_1.NpcModel(dbmodel);
                        this.initNpcModel(ach, player.id, ach.configId);
                        npcs.push(ach);
                    }
                    for (let ach of npcs) {
                        const key = ach.genKey();
                        store.save(ach, key);
                        store.saveKeyByType(player.id, dbkey, key);
                    }
                }
            });
        }
        initBattlefieldAsync() {
            return __awaiter(this, void 0, void 0, function* () {
                const store = this.store;
                const player = store.loadMe();
                const dbkey = "battlefield";
                let db = yield store.loadDbAsync(dbkey);
                let bm;
                if (db == null) {
                    bm = new battlefieldModel_1.BattlefieldModel();
                }
                else {
                    bm = new battlefieldModel_1.BattlefieldModel(db[0]);
                }
                const key = bm.genKey();
                store.save(bm, key);
                store.saveKeyByType(player.id, dbkey, key);
            });
        }
        initMonsterEmitterAsync() {
            return __awaiter(this, void 0, void 0, function* () {
                const store = this.store;
                const player = store.loadMe();
                const dbkey = "monster_emitter";
                let db = yield store.loadDbAsync(dbkey);
                let mem;
                if (db == null) {
                    mem = new monsterEmitterModel_1.MonsterEmitterModel();
                    mem.level = 1;
                    mem.configId = 1;
                }
                else {
                    mem = new monsterEmitterModel_1.MonsterEmitterModel(db[0]);
                }
                const key = mem.genKey();
                store.save(mem, key);
                store.saveKeyByType(player.id, dbkey, key);
            });
        }
        saveDbModelAsync(modelType) {
            return __awaiter(this, void 0, void 0, function* () {
                const store = this.serviceProvider.getService(modelStore_1.ModelStore);
                const player = store.loadMe();
                if (player == null) {
                    return null;
                }
                if (modelType == "player") {
                    return yield store.saveDbAsync(player);
                }
                const models = store.loadByType(player.id, modelType);
                if (models == null) {
                    return null;
                }
                const dbModels = new Array();
                for (let m of models) {
                    dbModels.push(m.toObject());
                }
                return yield store.saveRawDbAsync(modelType, dbModels);
            });
        }
        handleEvent(event, eventData) {
            switch (event) {
                case "monster_killed": {
                    let data = eventData;
                    this.updateAchievement(achievementModel_1.AchievementEvent.MonsterKilled, 1, data.configId);
                    if (data.isBoss) {
                        this.updateAchievement(achievementModel_1.AchievementEvent.BossKilled, 1, data.configId);
                    }
                    break;
                }
                case "difficulty_finished": {
                    let data = eventData;
                    this.updateAchievement(achievementModel_1.AchievementEvent.UnlockDifficulty, 1, data.level);
                    break;
                }
                case "damage_to_monster": {
                    let data = eventData;
                    this.updateAchievement(achievementModel_1.AchievementEvent.DamageToMonster, data);
                    break;
                }
                case "share": {
                    this.updateAchievement(achievementModel_1.AchievementEvent.SocialShare, 1);
                    break;
                }
            }
        }
        updateAchievement(event, count = 1, ...args) {
            const store = this.serviceProvider.getService(modelStore_1.ModelStore);
            const player = store.load("me");
            const configs = this.configUtil.getConfig(achievementConfiguration_1.AchievementConfiguration);
            const orderedList = configs.getOrderedList();
            const achievements = store.loadByType(player.id, "achievement");
            for (let ach of achievements) {
                if (ach.event != event) {
                    continue;
                }
                let config = configs.get(ach.configId);
                if (config.enable == 0) {
                    continue;
                }
                const params = config.params;
                if (params.length > args.length) {
                    continue;
                }
                let matchParams = true;
                for (let idx = 0; idx < params.length; idx++) {
                    if (params[idx] != args[idx]) {
                        matchParams = false;
                        break;
                    }
                }
                if (matchParams) {
                    ach.progress = ach.progress + count;
                    while (ach.progress >= config.times) {
                        let configChanged = false;
                        for (let entry of orderedList) {
                            if (entry.type == config.type && entry.times > config.times) {
                                config = entry;
                                configChanged = true;
                                break;
                            }
                        }
                        if (!configChanged) {
                            break;
                        }
                    }
                    ach.configId = config.id;
                }
            }
        }
        upgradeNpc(npcUid) {
            setTimeout(() => {
                const store = this.serviceProvider.getService(modelStore_1.ModelStore);
                const npc = store.load(npcUid);
                const player = store.load(playerModel_1.PlayerModel.genKey(npc.userId));
                if (player.gold >= npc.exp) {
                    player.gold -= npc.exp;
                    npc.level++;
                    this.updateAchievement(achievementModel_1.AchievementEvent.UpgradeNpcLevel, 1, npc.level);
                }
            });
        }
        buyNpc(userId, husbandId) {
            setTimeout(() => {
                const configUtil = this.serviceProvider.getService(configUtil_1.ConfigUtil);
                const store = this.serviceProvider.getService(modelStore_1.ModelStore);
                const config = configUtil.getConfig(npcConfiguration_1.NpcConfiguration).get(husbandId);
                const player = store.load(playerModel_1.PlayerModel.genKey(userId));
                if (player.gold < config.initGold) {
                    return;
                }
                player.gold -= config.initGold;
                let npc = new npcModel_1.NpcModel(userId + "_" + husbandId);
                this.initNpcModel(npc, userId, husbandId);
                let key = npc.genKey();
                const modelStore = this.serviceProvider.getService(modelStore_1.ModelStore);
                modelStore.save(npc, key);
                modelStore.saveKeyByType(userId, "npc", key);
                this.updateAchievement(achievementModel_1.AchievementEvent.BuyNpc, 1);
                this.notifyEventS("buyNpcSucceed", npc);
            });
        }
        initNpcModel(npc, userId, configId) {
            const config = this.configUtil.getConfig(npcConfiguration_1.NpcConfiguration).get(configId);
            const skills = this.configUtil.getConfig(npcDefaultSkillConfiguration_1.NpcDefaultSkillConfiguration).get(configId);
            npc.userId = userId;
            npc.baseAttack = config.atk;
            npc.baseExp = config.exp;
            npc.speed = config.spd;
            npc.diceNum = config.diceNum;
            npc.diceFaces = config.diceFaces;
            npc.criticalRate = config.criRate;
            npc.criticalRateParams = config.criRateParams;
            npc.skillRate = config.skillRate;
            npc.configId = configId;
            if (skills != null) {
                for (let skill of skills) {
                    let skillModel = new skillModel_1.SkillModel(userId, configId, skill.skillId, skill.conditionType);
                    npc.addSkill(skillModel);
                }
            }
        }
        saveAllDbAsync(syncNow = false) {
            return __awaiter(this, void 0, void 0, function* () {
                if (this.recoveringAccount) {
                    return;
                }
                const now = Date.now();
                this.syncRemote = syncNow || (now - this.lastSyncRemote > 30000);
                if (this.syncRemote) {
                    this.lastSyncRemote = now;
                }
                const typesToSave = ["player", "achievement", "npc", "battlefield", "monster_emitter"];
                const results = {};
                for (let t of typesToSave) {
                    const result = yield this.saveDbModelAsync(t);
                    if (result != null && this.syncRemote) {
                        results[t] = JSON.stringify(result);
                    }
                }
                if (this.syncRemote) {
                    this.platform.updatePlayerDataAsync(results).then();
                }
            });
        }
        recoverAccountAsync(newCustomId) {
            return __awaiter(this, void 0, void 0, function* () {
                this.recoveringAccount = true;
                const userData = yield this.platform.loginAndFetchUserDataAsync(newCustomId);
                if (userData == null) {
                    this.recoveringAccount = false;
                    return false;
                }
                const store = this.store;
                const keys = Object.keys(userData);
                for (let key of keys) {
                    const temp = JSON.parse(userData[key].Value);
                    if (key == "player") {
                        key = store.loadMe().genKey();
                    }
                    yield store.saveRawDbAsync(key, temp);
                }
                return true;
            });
        }
    }
    exports.PlayerLogic = PlayerLogic;
});
//# sourceMappingURL=playerLogic.js.map