var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
define(["require", "exports", "./scenes/entryScene", "./scenes/baseScene", "./utils/configUtil", "./models/modelStore", "./utils/networkUtil", "./serviceProvider", "./utils/engineUtil", "./configs/npcConfiguration", "./configs/monsterConfiguration", "./configs/monsterEmitterConfiguration", "./ui/characters/characterFactory", "./configs/skillConfiguration", "./configs/npcDefaultSkillConfiguration", "./scenes/battle/skills/skillFactory", "./configs/spritesheetConfiguration", "./configs/buffConfiguration", "./configs/emotionConfiguration", "./configs/damageCalculationConfiguration", "./scenes/battle/projectile/projectileFactory", "./utils/browserUtil", "./configs/projectileConfiguration", "./utils/androidUtil", "./configs/difficultyConfiguration", "./configs/achievementConfiguration", "./utils/i18n", "./utils/fontUtil", "./utils/logger", "./configs/gameConfiguration", "refs"], function (require, exports, entryScene_1, baseScene_1, configUtil_1, modelStore_1, networkUtil_1, serviceProvider_1, engineUtil_1, npcConfiguration_1, monsterConfiguration_1, monsterEmitterConfiguration_1, characterFactory_1, skillConfiguration_1, npcDefaultSkillConfiguration_1, skillFactory_1, spritesheetConfiguration_1, buffConfiguration_1, emotionConfiguration_1, damageCalculationConfiguration_1, projectileFactory_1, browserUtil_1, projectileConfiguration_1, androidUtil_1, difficultyConfiguration_1, achievementConfiguration_1, i18n_1, fontUtil_1, logger_1, gameConfiguration_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class BillionsGame {
        constructor() {
            this.game = new Phaser.Game(BillionsGame.targetWidth, BillionsGame.targetHeight, Phaser.CANVAS, 'content', {
                create: this.create
            });
        }
        create() {
            const g = sGame.game;
            g.scale.scaleMode = Phaser.ScaleManager.USER_SCALE;
            let w = g.canvas.parentElement.offsetWidth;
            let ww = Math.max(window.innerWidth, document.documentElement.clientWidth);
            let wh = Math.max(window.innerHeight, document.documentElement.clientHeight);
            let scaleX = w / g.width;
            g.scale.setUserScale(scaleX, scaleX, 0, 0, false, false);
            g.scale.setGameSize(BillionsGame.targetWidth, BillionsGame.targetWidth / ww * wh);
            g.add.text(0, 0, ".", { font: "1px GenShinGothic", fill: "#FFFFFF" });
            g.add.text(0, 0, ".", { font: "1px PixelMplus12-Regular", fill: "#FFFFFF" });
            g.input.maxPointers = 1;
            if (g.device.cordova) {
                document.addEventListener("deviceready", () => {
                    sGame.createAsync().then();
                }, false);
            }
            else {
                sGame.createAsync().then();
            }
        }
        createAsync() {
            return __awaiter(this, void 0, void 0, function* () {
                const g = this.game;
                const sp = new serviceProvider_1.ServiceProvider();
                const configUtil = yield this.initConfigAsync();
                const platform = yield this.initPlatformAsync(sp, g, configUtil);
                const language = platform.language;
                yield i18n_1.I18n.initAsync(g, language);
                if (language.startsWith("en")) {
                    fontUtil_1.FontUtil.DFW = "bold";
                }
                sp.addSingleton(configUtil);
                sp.addSingleton(new modelStore_1.ModelStore());
                sp.addSingleton(new networkUtil_1.NetworkUtil());
                sp.addSingleton(new characterFactory_1.CharacterFactory().setGame(g).setConfigUtil(configUtil));
                sp.addSingleton(new skillFactory_1.SkillFactory());
                sp.addSingleton(new projectileFactory_1.ProjectileFactory());
                g.load.path = "img/";
                g.stage.disableVisibilityChange = true;
                if (!platform.isMobile) {
                    g.time.advancedTiming = true;
                }
                yield engineUtil_1.EngineUtil.loadStartAsync(g);
                this.enterGame(sp, g);
            });
        }
        initPlatformAsync(sp, g, configUtil) {
            return __awaiter(this, void 0, void 0, function* () {
                let platformUtil = null;
                if (g.device.cordova) {
                    if (g.device.iOS) {
                    }
                    else {
                        platformUtil = new androidUtil_1.AndroidUtil();
                    }
                    new logger_1.PlayFabLogger();
                }
                else {
                    platformUtil = new browserUtil_1.BrowserUtil();
                    new logger_1.ConsoleLogger();
                }
                platformUtil.configUtil = configUtil;
                platformUtil.serviceProvider = sp;
                yield platformUtil.initAsync();
                sp.addSingletonByName("platformUtil", platformUtil);
                return platformUtil;
            });
        }
        initConfigAsync() {
            return __awaiter(this, void 0, void 0, function* () {
                let instance = new configUtil_1.ConfigUtil();
                instance.game = this.game;
                instance.addEncryptedConfig(new gameConfiguration_1.GameConfiguration());
                yield engineUtil_1.EngineUtil.loadStartAsync(this.game);
                instance.addConfig(new npcConfiguration_1.NpcConfiguration());
                instance.addConfig(new monsterConfiguration_1.MonsterConfiguration());
                instance.addConfig(new monsterEmitterConfiguration_1.MonsterEmitterConfiguration());
                instance.addConfig(new skillConfiguration_1.SkillConfiguration());
                instance.addConfig(new npcDefaultSkillConfiguration_1.NpcDefaultSkillConfiguration());
                instance.addConfig(new spritesheetConfiguration_1.SpritesheetConfiguration());
                instance.addConfig(new buffConfiguration_1.BuffConfiguration());
                instance.addConfig(new emotionConfiguration_1.EmotionConfiguration());
                instance.addConfig(new damageCalculationConfiguration_1.DamageCalculationConfiguration());
                instance.addConfig(new projectileConfiguration_1.ProjectileConfiguration());
                instance.addConfig(new difficultyConfiguration_1.DifficultyConfiguration());
                instance.addConfig(new achievementConfiguration_1.AchievementConfiguration());
                return instance;
            });
        }
        enterGame(sp, g) {
            let store = sp.getService(modelStore_1.ModelStore);
            store.initAsync().then(() => {
                let entryScene = new entryScene_1.EntryScene(sp);
                new baseScene_1.SceneTransition(g, entryScene).start(false);
            });
        }
    }
    BillionsGame.targetWidth = 640;
    BillionsGame.targetHeight = 1136;
    let sGame = new BillionsGame();
});
//# sourceMappingURL=main.js.map