define(["require", "exports", "./baseModel"], function (require, exports, baseModel_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class BattlefieldModel extends baseModel_1.BaseModel {
        constructor(source) {
            super();
            this._monsterRecords = new Map();
            this._taps = 0;
            this._rush = false;
            if (source != null) {
                const records = source.monsterRecords;
                for (let record of records) {
                    this._monsterRecords.set(record.id, record);
                }
            }
        }
        get rush() {
            return this._rush;
        }
        set rush(value) {
            this._rush = value;
        }
        get taps() {
            return this._taps;
        }
        set taps(value) {
            if (value != this._taps) {
                this._taps = value;
                this.notifyPropertyChangeS("taps", value);
            }
        }
        recordLevel(id, level) {
            let info = this.getMonsterRecord(id, true);
            if (info.maxLevel < level) {
                info.maxLevel = level;
            }
        }
        recordKilled(id) {
            let info = this.getMonsterRecord(id, true);
            info.killed += 1;
        }
        getMonsterRecord(id, create = false) {
            if (this._monsterRecords.has(id)) {
                return this._monsterRecords.get(id);
            }
            if (create) {
                let record = {
                    id: id,
                    maxLevel: 1,
                    killed: 0
                };
                this._monsterRecords.set(id, record);
                return record;
            }
            return null;
        }
        toObject() {
            let obj1 = super.toObject();
            let arr = Array.from(this._monsterRecords.values());
            let obj2 = {
                monsterRecords: arr
            };
            return Object.assign({}, obj1, obj2);
        }
        notifyPropertyChangeS(event, eventData) {
            super.notifyPropertyChangeS(event, eventData);
        }
        onPropertyChangedS(event, caller, listener) {
            return super.onPropertyChangedS(event, caller, listener);
        }
        static genKey() {
            return BattlefieldModel._name;
        }
        genKey() {
            return BattlefieldModel.genKey();
        }
    }
    BattlefieldModel._name = "battlefield";
    exports.BattlefieldModel = BattlefieldModel;
});
//# sourceMappingURL=battlefieldModel.js.map