var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
define(["require", "exports", "localforage"], function (require, exports, lf) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class ModelStore {
        constructor() {
            this._cache = {};
            this._keysCache = {};
        }
        load(key) {
            if (!this._cache.hasOwnProperty(key)) {
                return null;
            }
            return this._cache[key];
        }
        loadMe() {
            return this.load("me");
        }
        loadByKeys(keys) {
            let ret = [];
            for (let key of keys) {
                if (this._cache.hasOwnProperty(key)) {
                    ret.push(this._cache[key]);
                }
            }
            return ret;
        }
        save(m, key) {
            if (key == null) {
                key = m.genKey();
            }
            if (this._cache.hasOwnProperty(key)) {
                throw new Error("key already exist, key=" + key);
            }
            this._cache[key] = m;
        }
        saveMe(m) {
            this.save(m, "me");
        }
        clearAllKeys(userId, modelType) {
            let k = userId + "." + modelType;
            if (this._keysCache.hasOwnProperty(k)) {
                this._keysCache[k] = [];
            }
        }
        saveKeyByType(userId, modelType, key) {
            let k = userId + "." + modelType;
            let arr;
            if (this._keysCache.hasOwnProperty(k)) {
                arr = this._keysCache[k];
            }
            else {
                arr = [];
                this._keysCache[k] = arr;
            }
            arr.push(key);
        }
        loadByType(userId, modelType) {
            let k = userId + "." + modelType;
            if (this._keysCache.hasOwnProperty(k)) {
                return this.loadByKeys(this._keysCache[k]);
            }
            return [];
        }
        initAsync() {
            return __awaiter(this, void 0, void 0, function* () {
                this._db = lf.createInstance({ name: 'TrillionsOfMonsters' });
                try {
                    yield this._db.ready();
                }
                catch (e) {
                    console.error(e);
                }
            });
        }
        loadDbAsync(key) {
            return __awaiter(this, void 0, void 0, function* () {
                try {
                    return yield this._db.getItem(key);
                }
                catch (e) {
                    console.error(`loadDb [${key}] failed`, e);
                }
            });
        }
        saveDbAsync(model, key) {
            return __awaiter(this, void 0, void 0, function* () {
                if (key == null) {
                    key = model.genKey();
                }
                try {
                    model.dirty = false;
                    return yield this._db.setItem(key, model.toObject());
                }
                catch (e) {
                    console.error(`saveDb [${key}] failed`, e);
                }
                return null;
            });
        }
        saveRawDbAsync(key, item) {
            return __awaiter(this, void 0, void 0, function* () {
                try {
                    return yield this._db.setItem(key, item);
                }
                catch (e) {
                    console.error(`saveRawDb [${key}] failed`, e);
                }
                return null;
            });
        }
    }
    exports.ModelStore = ModelStore;
});
//# sourceMappingURL=modelStore.js.map