define(["require", "exports", "./baseModel"], function (require, exports, baseModel_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class NpcModel extends baseModel_1.BaseModel {
        constructor(source) {
            super();
            this._skills = [];
            if (typeof source === 'string') {
                this._uid = source;
                this._level = 1;
            }
            else {
                for (let n in source) {
                    this['_' + n] = source[n];
                }
            }
        }
        get skillRate() {
            return this._skillRate;
        }
        set skillRate(value) {
            this._skillRate = value;
        }
        get speed() {
            return this._speed;
        }
        set speed(value) {
            this._speed = value;
        }
        get userId() {
            return this._userId;
        }
        get exp() {
            const lv = this.level + 1;
            return Math.ceil((0.2002 * lv * lv - 0.4304 * lv + 1.2302) * this._baseExp);
        }
        set baseExp(value) {
            this._baseExp = value;
        }
        get diceNum() {
            return this._diceNum;
        }
        set diceNum(value) {
            this._diceNum = value;
        }
        get diceFaces() {
            return this._diceFaces;
        }
        set diceFaces(value) {
            this._diceFaces = value;
        }
        get criticalRate() {
            return this._criticalRate + Math.floor((this._level - 1) / this._criticalRateParams[0]);
        }
        set criticalRate(value) {
            this._criticalRate = value;
        }
        get attack() {
            return Math.ceil((0.7222323 + 0.2171933 * this._level + 0.002680284 * this._level * this._level) * this._baseAttack);
        }
        get minAttack() {
            return this.attack + this._diceNum;
        }
        get maxAttack() {
            const atk = this.attack;
            return atk + this._diceNum * (this._diceFaces < 1 ? Math.ceil(atk * this._diceFaces) : this._diceFaces);
        }
        set criticalRateParams(value) {
            this._criticalRateParams = value;
        }
        set baseAttack(value) {
            this._baseAttack = value;
        }
        get level() {
            return this._level;
        }
        set level(value) {
            if (this.level != value) {
                this._level = value;
                this.notifyPropertyChangeS("level", value);
            }
        }
        get uid() {
            return this._uid;
        }
        get configId() {
            return this._configId;
        }
        set configId(value) {
            this._configId = value;
        }
        set userId(value) {
            this._userId = value;
        }
        get skills() {
            return this._skills;
        }
        addSkill(skill) {
            this._skills.push(skill);
        }
        toObject() {
            let obj1 = super.toObject();
            let obj2 = {
                uid: this._uid,
                userId: this._userId,
                level: this._level,
                configId: this._configId
            };
            return Object.assign({}, obj1, obj2);
        }
        notifyPropertyChangeS(event, eventData) {
            super.notifyPropertyChangeS(event, eventData);
        }
        onPropertyChangedS(event, caller, listener) {
            return super.onPropertyChangedS(event, caller, listener);
        }
        genKey() {
            return NpcModel._name + "_" + this._uid;
        }
    }
    NpcModel._name = "npc";
    exports.NpcModel = NpcModel;
});
//# sourceMappingURL=npcModel.js.map