var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
define(["require", "exports", "../ui/feGameUI", "../logics/playerLogic", "../logics/guideLogic", "../utils/configUtil", "../models/modelStore", "../utils/networkUtil", "../utils/engineUtil", "../ui/characters/characterUI", "../utils/intervalTimer", "../ui/windows/textMsgBox", "../utils/i18n"], function (require, exports, feGameUI_1, playerLogic_1, guideLogic_1, configUtil_1, modelStore_1, networkUtil_1, engineUtil_1, characterUI_1, intervalTimer_1, textMsgBox_1, i18n_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class SceneTransition {
        constructor(game, scene) {
            this._scene = scene;
            this._scene.game = game;
        }
        start(useTranScene = true) {
            let g = this._scene.game;
            let curScene = g.state.getCurrentState();
            if (useTranScene) {
                g.add.tween(curScene.safezone).to({ alpha: 0 }, 500, Phaser.Easing.Default, true).onComplete.addOnce(() => {
                    this._scene.preloadRes();
                    g.input.enabled = false;
                    g.load.onLoadComplete.addOnce(() => {
                        g.state.add(this._scene.name, this._scene, true);
                        g.input.enabled = true;
                    });
                    engineUtil_1.EngineUtil.loadStart(g);
                });
            }
            else {
                this._scene.preloadRes();
                g.input.enabled = false;
                g.load.onLoadComplete.addOnce(() => {
                    g.state.add(this._scene.name, this._scene, true);
                    g.input.enabled = true;
                });
            }
            engineUtil_1.EngineUtil.loadStart(g);
        }
    }
    exports.SceneTransition = SceneTransition;
    class WindowAction {
        constructor(win, state) {
            this.win = win;
            this.state = state;
        }
    }
    class BaseScene extends Phaser.State {
        constructor(name, sp) {
            super();
            this._disposables = [];
            this._logics = {};
            this._windowsStack = Array();
            this._pendingWindowAction = [];
            this._name = name;
            this._serviceProvider = sp;
            this.feUI = new feGameUI_1.FeGameUI(this);
            this.charUI = new characterUI_1.CharacterUI(this);
        }
        get serviceProvider() {
            return this._serviceProvider;
        }
        get modelStore() {
            return this._serviceProvider.getService(modelStore_1.ModelStore);
        }
        get configUtil() {
            return this._serviceProvider.getService(configUtil_1.ConfigUtil);
        }
        get network() {
            return this._serviceProvider.getService(networkUtil_1.NetworkUtil);
        }
        get platform() {
            return this._serviceProvider.getServiceByName("platformUtil");
        }
        get name() {
            return this._name;
        }
        get safeWidth() {
            return this._safeWidth;
        }
        get safeHeight() {
            return this._safeHeight;
        }
        get safezone() {
            return this._safezone;
        }
        addLogic(logic) {
            this._logics[logic.name] = logic;
            logic.serviceProvider = this._serviceProvider;
            logic.registerEvents();
            return this;
        }
        getLogic(c) {
            let l = this._logics[c.name];
            if (l == null) {
                return null;
            }
            return l;
        }
        init() {
            this.addLogic(new playerLogic_1.PlayerLogic());
            this.addLogic(new guideLogic_1.GuideLogic());
        }
        create() {
            let game_ratio = this.game.scale.aspectRatio;
            this._safezone = this.game.add.group();
            this._safeWidth = this.game.width;
            this._safeHeight = this.game.width / BaseScene._safeZoneRatio;
            if (game_ratio > BaseScene._safeZoneRatio) {
                let scale = BaseScene._safeZoneRatio / game_ratio;
                if (scale > 0.95 && scale < 1.05) {
                    this._safezone.scale.set(1, scale);
                }
                else {
                    this._safezone.scale.set(scale);
                    this._safezone.position.set((this.game.width - this.game.width * scale) / 2, 0);
                }
            }
            else if (game_ratio < BaseScene._safeZoneRatio) {
                let scale = game_ratio / BaseScene._safeZoneRatio;
                this._safezone.position.set(0, (this.game.height - this.game.height * scale) / 2);
            }
            this.addDisposable(this.platform.on("back_btn", this, this.onBackButton));
            this.createAsync().then(() => {
                this._saveDbTimer = new intervalTimer_1.IntervalTimer(5000);
                this._saveDbTimer.reset();
            });
        }
        addDisposable(disp) {
            this._disposables.push(disp);
        }
        shutdown() {
            this.game.state.remove(this.key);
            for (let d of this._disposables) {
                d.dispose();
            }
            this._disposables = [];
        }
        render() {
            if (this.platform.isMobile) {
                return;
            }
            this.game.debug.text('FPS: ' + this.game.time.fps || 'FPS: --', 5, 15, "#00ff00");
        }
        preloadRes() {
            this.feUI.preloadRes();
            this.charUI.preloadResAsync().then();
        }
        checkAndLoadImage(key, path, callback, callbackContext) {
            engineUtil_1.EngineUtil.checkAndLoadImage(this.game, key, path == null ? key : path, callback, callbackContext);
        }
        checkAndLoadImageAsync(key, path) {
            return __awaiter(this, void 0, void 0, function* () {
                this.checkAndLoadImage(key, path);
                yield engineUtil_1.EngineUtil.loadStartAsync(this.game);
                return key;
            });
        }
        checkAndLoadImagesAsync(keys) {
            return __awaiter(this, void 0, void 0, function* () {
                for (let key of keys) {
                    this.checkAndLoadImage(key);
                }
                yield engineUtil_1.EngineUtil.loadStartAsync(this.game);
                return keys;
            });
        }
        checkAndLoadSpriteSheet(key, path, frameWidth, frameHeight, margin, spacing, callback, callbackContext) {
            engineUtil_1.EngineUtil.checkAndLoadSpriteSheet(this.game, key, path == null ? key : path, frameWidth, frameHeight, margin, spacing, callback, callbackContext);
        }
        onBackButton() {
            if (this._windowsStack.length > 0) {
                const curWin = this._windowsStack[this._windowsStack.length - 1];
                if (curWin.enableBackBtn) {
                    curWin.closeMe();
                }
                return;
            }
            let msgbox = new textMsgBox_1.TextMsgBox(this, i18n_1.I18n.t("menu.quit.game"));
            msgbox.showConfirmButton = true;
            msgbox.on("ok", this, () => {
                this.saveAllDbAsync().then(() => {
                    this.platform.exitApp();
                });
            });
            this.popupWindow(msgbox);
        }
        setGrayLayerVisible(visible) {
            if (this._grayLayer == null) {
                this._grayLayer = this.game.add.graphics(-this.safeWidth / 2, -this.safeHeight / 2, this.safezone);
                this._grayLayer.beginFill(0x000000);
                this._grayLayer.drawRect(0, 0, this.safeWidth * 2, this.safeHeight * 2);
                this._grayLayer.endFill();
                this._grayLayer.alpha = 0.4;
                this._grayLayer.inputEnabled = true;
            }
            this._grayLayer.visible = visible;
        }
        startLoading() {
            if (this.loadingWindow == null) {
                const msgbox = new textMsgBox_1.TextMsgBox(this, i18n_1.I18n.t("menu.loading"));
                msgbox.hideAllButton = true;
                msgbox.enableBackBtn = false;
                this.popupWindow(msgbox);
                this.loadingWindow = msgbox;
            }
        }
        stopLoading() {
            if (this.loadingWindow != null) {
                this.closeWindow(this.loadingWindow);
                this.loadingWindow = null;
            }
        }
        popupWindow(window) {
            this.stopLoading();
            this._pendingWindowAction.push(new WindowAction(window, 1));
        }
        closeWindow(window) {
            this._pendingWindowAction.push(new WindowAction(window, 2));
        }
        doPopupWindow(window) {
            window.preloadRes();
            this.game.input.enabled = false;
            this.game.load.onLoadComplete.addOnce(() => {
                window.initView();
                const ww = window.winWidth == null ? window.width : window.winWidth;
                const wh = window.winHeight == null ? window.height : window.winHeight;
                window.pivot.set(ww / 2, wh / 2);
                window.position.set(this.safeWidth / 2, this.safeHeight / 2);
                this.safezone.add(window);
                this._windowsStack.push(window);
                this.setGrayLayerVisible(true);
                this.adjustGrayLayerZOrder();
                window.onShow();
                this.game.input.enabled = true;
            }, this);
            engineUtil_1.EngineUtil.loadStart(this.game);
        }
        doCloseWindow(window) {
            window.destroy(true);
            this._windowsStack.pop();
            if (this._windowsStack.length == 0) {
                this.setGrayLayerVisible(false);
            }
            else {
                this.adjustGrayLayerZOrder();
            }
        }
        saveAllDbAsync() {
            return __awaiter(this, void 0, void 0, function* () {
                const logic = this.getLogic(playerLogic_1.PlayerLogic);
                yield logic.saveAllDbAsync();
            });
        }
        update(game) {
            super.update(game);
            const action = this._pendingWindowAction.shift();
            if (action != null) {
                if (action.state == 1)
                    this.doPopupWindow(action.win);
                if (action.state == 2)
                    this.doCloseWindow(action.win);
            }
            else if (this._saveDbTimer != null && this._saveDbTimer.checkAndReset()) {
                this.saveAllDbAsync().then();
            }
        }
        adjustGrayLayerZOrder() {
            if (this._windowsStack.length > 0) {
                this.safezone.bringToTop(this._grayLayer);
                this.safezone.bringToTop(this._windowsStack[this._windowsStack.length - 1]);
            }
        }
    }
    BaseScene._safeZoneRatio = 9 / 16;
    exports.BaseScene = BaseScene;
});
//# sourceMappingURL=baseScene.js.map