var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
define(["require", "exports", "../../baseGameGroup", "./monsterEmitter", "../../configs/battleFieldConfiguration", "../../configs/monsterConfiguration", "../../configs/npcConfiguration", "../../ui/characters/characterFactory", "../../configs/spritesheetConfiguration", "../../utils/eventbus", "../../logics/playerLogic"], function (require, exports, baseGameGroup_1, monsterEmitter_1, battleFieldConfiguration_1, monsterConfiguration_1, npcConfiguration_1, characterFactory_1, spritesheetConfiguration_1, eventbus_1, playerLogic_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class Battlefield extends baseGameGroup_1.BaseGameGroup {
        constructor(scene, width, height, player) {
            super(scene);
            this._eventBus = new eventbus_1.Eventbus();
            this._npcs = [];
            this._monsters = new Map();
            this._pendingActions = [];
            this._actions = [];
            this._addingCharacter = false;
            this._totalDt = 0;
            const store = scene.modelStore;
            this._playerModel = player;
            this._boundingBox = new Phaser.Rectangle(0, 150, width, height - 150);
            const modelKey = "battlefield";
            this._model = store.load(modelKey);
            this._xUnitToPixel = this._boundingBox.width / battleFieldConfiguration_1.BattleFieldConfiguration.GridSize;
            this._yUnitToPixel = this._boundingBox.height / battleFieldConfiguration_1.BattleFieldConfiguration.GridSize;
            let graphics = scene.game.add.graphics(0, 0, this);
            graphics.alpha = 0;
            graphics.beginFill(0x000000).drawRect(0, 0, width, height);
            graphics.inputEnabled = true;
            graphics.events.onInputUp.add(this.onBattlefieldTapped, this);
            const emitterModelKey = "monster_emitter";
            let emitterModel = store.load(emitterModelKey);
            this._emitter = new monsterEmitter_1.MonsterEmitter(emitterModel, scene.getLogic(playerLogic_1.PlayerLogic), scene.configUtil);
            this._emitter.start();
        }
        get emitter() {
            return this._emitter;
        }
        get playerModel() {
            return this._playerModel;
        }
        get npcs() {
            return this._npcs;
        }
        get monsters() {
            return this._monsters.values();
        }
        get model() {
            return this._model;
        }
        get xUnitToPixel() {
            return this._xUnitToPixel;
        }
        get yUnitToPixel() {
            return this._yUnitToPixel;
        }
        get boundingBox() {
            return this._boundingBox;
        }
        addNpc(npc) {
            return __awaiter(this, void 0, void 0, function* () {
                this._addingCharacter = true;
                const scene = this.getCurrentScene();
                const config = scene.configUtil.getConfig(npcConfiguration_1.NpcConfiguration);
                const npcConfig = config.get(npc.configId);
                const spritesheetConfig = scene.configUtil.getConfig(spritesheetConfiguration_1.SpritesheetConfiguration);
                const charFact = scene.serviceProvider.getService(characterFactory_1.CharacterFactory);
                const spritesheet = spritesheetConfig.getConfig(npcConfig.battlePathId);
                const character = yield charFact.createNpc(0, 0, spritesheet.path, spritesheet.dimension, npcConfig);
                character.scale.set(0.6);
                character.model = npc;
                character.field = this;
                character.anchor.set(1, 0.5);
                const npcCount = this._npcs.length;
                const maxPerColumn = 6;
                const row = npcCount % maxPerColumn;
                const column = Math.floor(npcCount / maxPerColumn);
                const centerY = this._boundingBox.y + this._boundingBox.height / 2 + column * character.height / 2;
                const posY = centerY + Math.ceil(row / 2) * character.height * (row % 2 == 0 ? 1 : -1);
                character.origPosition = new Phaser.Point(this._boundingBox.width - (character.width * 2 / 3) * column, posY);
                this.addChild(character);
                character.standby();
                this.game.add.tween(character).from({ x: this._boundingBox.width + character.width }, 500).start();
                this._npcs.push(character);
                this._addingCharacter = false;
            });
        }
        addMonsters(monsters) {
            return __awaiter(this, void 0, void 0, function* () {
                this._addingCharacter = true;
                const scene = this.getCurrentScene();
                const charFact = scene.serviceProvider.getService(characterFactory_1.CharacterFactory);
                const config = scene.configUtil.getConfig(monsterConfiguration_1.MonsterConfiguration);
                const spritesheetConfig = scene.configUtil.getConfig(spritesheetConfiguration_1.SpritesheetConfiguration);
                let npcCount = 0;
                for (let monster of monsters) {
                    const npcConfig = config.get(monster.configId);
                    const spritesheet = spritesheetConfig.getConfig(npcConfig.pathId);
                    const character = yield charFact.createMonster(0, 0, spritesheet.path, spritesheet.dimension, npcConfig);
                    character.model = monster;
                    character.field = this;
                    character.xUnitToPixel = this._xUnitToPixel;
                    character.yUnitToPixel = this._yUnitToPixel;
                    character.anchor.set(1, 0.5);
                    if (!monster.isBoss) {
                        character.scale.set(0.6);
                    }
                    const startY = (this._boundingBox.height - character.height) * Math.random() + this._boundingBox.y;
                    character.position.set(character.width, startY);
                    this.addChild(character);
                    this._model.recordLevel(monster.configId, monster.level);
                    character.walk("right");
                    npcCount++;
                    character.onPropertyChangedS("destroyed", this, (args) => {
                        const temp = this._monsters.get(monster.uid);
                        if (args == "attack") {
                            this.onMonsterKilled(temp);
                            this._emitter.onMonsterDestroyed(temp.model, true, this._monsters.size <= 1);
                        }
                        else {
                            this._emitter.onMonsterDestroyed(temp.model, false, this._monsters.size <= 1);
                        }
                        this._monsters.delete(monster.uid);
                    });
                    this._monsters.set(monster.uid, character);
                }
                this.sortMonsters();
                this._addingCharacter = false;
            });
        }
        sortMonsters() {
            const arr = Array.from(this._monsters.values());
            arr.sort((a, b) => {
                return a.y + a.height / 2 - b.y - b.height / 2;
            });
            for (let m of arr) {
                this.bringToTop(m);
            }
        }
        onBattlefieldTapped() {
            const m = this._model;
            m.taps = m.taps + 1;
        }
        onMonsterKilled(monster) {
            const pm = this._playerModel;
            const mm = monster.model;
            if (pm != null) {
                pm.gold = pm.gold + mm.gold;
                pm.magicEssence = pm.magicEssence + mm.magicEssence;
                this._model.recordKilled(monster.model.configId);
            }
            this.getCurrentScene().getLogic(playerLogic_1.PlayerLogic).handleEvent("monster_killed", mm);
            this.notifyPropertyChangeS("MonsterKilled", monster);
        }
        update() {
            super.update();
            const physicsElapsedMs = this.game.time.physicsElapsedMS;
            this._totalDt += physicsElapsedMs;
            if (this._model.rush && this._playerModel.gauge > 0) {
                this._playerModel.gauge -= physicsElapsedMs;
            }
            while (this._totalDt >= battleFieldConfiguration_1.BattleFieldConfiguration.FrameInterval) {
                this.updateFrame();
                const aliveActions = new Array();
                for (let action of this._actions) {
                    if (!action.finished) {
                        action.updateFrame();
                        aliveActions.push(action);
                    }
                }
                while (this._pendingActions.length > 0) {
                    aliveActions.push(this._pendingActions.shift());
                }
                this._actions = aliveActions;
                for (let npc of this._npcs) {
                    npc.updateFrame();
                }
                for (let m of this._monsters.values()) {
                    m.updateFrame();
                }
                this._totalDt -= battleFieldConfiguration_1.BattleFieldConfiguration.FrameInterval;
            }
        }
        addAction(action) {
            this._pendingActions.push(action);
        }
        updateFrame() {
            const bm = this._model;
            for (let i = 0; i < this._npcs.length; i++) {
                const npc = this._npcs[i];
                const result = npc.decideAction(bm);
                if (result.action != null) {
                    this._actions.push(result.action);
                }
            }
            if (this._npcs.length == 0 || this._monsters.size == 0) {
                bm.taps = 0;
            }
            if (this._addingCharacter) {
                return;
            }
            const emitter = this._emitter;
            const monsters = emitter.tryEmit();
            if (monsters != null && monsters.length > 0) {
                this.addMonsters(monsters).then();
            }
        }
        notifyPropertyChangeS(name, value) {
            this._eventBus.emit(name, value);
        }
        onPropertyChangedS(evt, caller, listener) {
            return this._eventBus.on(evt, (event) => {
                listener.call(caller, event);
            });
        }
    }
    exports.Battlefield = Battlefield;
});
//# sourceMappingURL=battlefield.js.map