define(["require", "exports", "../characterAction", "../../../ui/characters/monsterCharacter", "../../../utils/engineUtil", "../../../configs/spritesheetConfiguration", "../../../configs/battleFieldConfiguration"], function (require, exports, characterAction_1, monsterCharacter_1, engineUtil_1, spritesheetConfiguration_1, battleFieldConfiguration_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class Projectile extends characterAction_1.CharacterAction {
        constructor(field, config, target, actor, damage) {
            super(field);
            this._traceTarget = true;
            this._state = "res";
            this._target = target;
            this.actor = actor;
            this.config = config;
            this.damage = damage;
        }
        set traceTarget(value) {
            this._traceTarget = value;
        }
        get traceTarget() {
            return this._traceTarget;
        }
        get target() {
            return this._target;
        }
        get effectKey() {
            return Projectile.ResPrefix + this.config.id;
        }
        updateFrame() {
            if (this._state == "res") {
                this._state = "init";
                this.preloadRes();
                this.game.load.onLoadComplete.addOnce(this.init, this);
                engineUtil_1.EngineUtil.loadStart(this.game);
            }
            else if (this._state == "done") {
                this.doUpdateFrame();
                if (this.finished && this.damage) {
                    if (this._target instanceof monsterCharacter_1.MonsterCharacter && this._target.alive) {
                        this._target.suffer([this.damage]);
                    }
                    this.damage = null;
                }
            }
        }
        init() {
            this._state = "done";
        }
        preloadRes() {
            const f = this.field;
            const bf = f.getCurrentScene();
            const spritesheetConfig = bf.configUtil.getConfig(spritesheetConfiguration_1.SpritesheetConfiguration);
            const conf = this.config;
            if (conf.pathId) {
                const spritesheet = spritesheetConfig.getConfig(conf.pathId);
                f.checkAndLoadSpriteSheet(this.effectKey, spritesheet.path, spritesheet.dimension[0], spritesheet.dimension[1]);
            }
        }
    }
    Projectile.ResPrefix = "projectile_effect_";
    exports.Projectile = Projectile;
    class Arrow extends Projectile {
        doUpdateFrame() {
            if (this.effect == null) {
                let effect = this.game.make.sprite(this.actor.x - this.actor.width, this.actor.y, this.effectKey);
                this.effect = effect;
                effect.angle = this.config.initAngle + this.getAngleToTarget();
                effect.anchor.set(0.5, 0.5);
                effect.animations.add("attack", this.config.animation);
                effect.animations.play("attack", 15, true);
                this.field.addChild(effect);
                this.xDistance = effect.x - this.target.x;
                this.yDistance = Math.abs(effect.y - this.target.y);
                if (this.target instanceof monsterCharacter_1.MonsterCharacter) {
                    this.totalFrames = Math.ceil(this.xDistance / this.field.xUnitToPixel / (this.target.moveSpeed + this.config.xSpeed) * battleFieldConfiguration_1.BattleFieldConfiguration.FramePerSecond);
                }
                else {
                    this.totalFrames = Math.ceil(this.xDistance / this.field.xUnitToPixel / this.config.xSpeed * battleFieldConfiguration_1.BattleFieldConfiguration.FramePerSecond);
                }
                this.prevAngle = effect.angle;
            }
            if (this.effect.x > this.target.x) {
                this.effect.x -= this.xDistance / this.totalFrames;
                if (this.effect.y > this.target.y) {
                    this.effect.y -= this.yDistance / this.totalFrames;
                }
                else {
                    this.effect.y += this.yDistance / this.totalFrames;
                }
                if (this.traceTarget) {
                    const newAngle = this.config.initAngle + this.getAngleToTarget();
                    const maxAngleDiff = 0.3 * this.config.xSpeed / 32;
                    if (Math.abs(this.prevAngle - newAngle) > maxAngleDiff) {
                        if (this.prevAngle < newAngle) {
                            this.effect.angle = this.prevAngle + maxAngleDiff;
                        }
                        else {
                            this.effect.angle = this.prevAngle - maxAngleDiff;
                        }
                    }
                    else {
                        this.effect.angle = newAngle;
                    }
                    this.prevAngle = this.effect.angle;
                }
            }
            else {
                this.finished = true;
                this.effect.pendingDestroy = true;
            }
        }
        getAngleToTarget() {
            return Math.atan2(this.effect.y - this.target.y, this.effect.x - this.target.x) * 180 / Math.PI;
        }
    }
    class ProjectileFactory {
        create(field, config, target, actor, damage) {
            let ret = null;
            switch (config.type) {
                case 1:
                    ret = new Arrow(field, config, target, actor, damage);
                    break;
            }
            return ret;
        }
    }
    exports.ProjectileFactory = ProjectileFactory;
});
//# sourceMappingURL=projectileFactory.js.map