define(["require", "exports", "../../../configs/projectileConfiguration", "../projectile/projectileFactory", "./remoteAttack"], function (require, exports, projectileConfiguration_1, projectileFactory_1, remoteAttack_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class AstralArrow extends remoteAttack_1.RemoteAttack {
        doUpdateFrame() {
            if (this.arrowAction == null) {
                let targets = this.findTargets();
                if (targets.length > 0) {
                    const target = targets[Math.floor(targets.length / 2)];
                    const scene = this.field.getCurrentScene();
                    const proj = scene.serviceProvider.getService(projectileFactory_1.ProjectileFactory);
                    const configEntry = scene.configUtil.getConfig(projectileConfiguration_1.ProjectileConfiguration).getConfig(this.config.projectileId);
                    this.dest = new Phaser.Point(target.x, target.y - this.npc.height);
                    this.arrowAction = proj.create(this.field, configEntry, this.dest, this.npc);
                    this.arrowAction.traceTarget = false;
                    this.field.addAction(this.arrowAction);
                    this.showActorEffect(target);
                }
            }
            else if (this.arrowAction.finished) {
                const maxHorizonDistance = 112;
                const targets = this.field.monsters;
                const xUnit = this.dest.x / this.field.xUnitToPixel, yUnit = this.dest.y / this.field.yUnitToPixel;
                for (let m of targets) {
                    if (m.xUnit > xUnit + maxHorizonDistance || m.xUnit < xUnit - maxHorizonDistance) {
                        continue;
                    }
                    if (m.yUnit > yUnit + 128 || m.yUnit < yUnit) {
                        continue;
                    }
                    this.applyDamage(m, [this.calculateDamage(m)]);
                }
                this.showTargetEffect(null);
                this.finished = true;
            }
        }
        showTargetEffect(target) {
            let effect = this.game.make.sprite(this.dest.x, this.dest.y, this.effectKey);
            effect.anchor.set(0.5, 0);
            effect.animations.add("attack", this.config.animation);
            effect.animations.play("attack", 15, false, true);
            this.field.addChild(effect);
        }
    }
    exports.AstralArrow = AstralArrow;
});
//# sourceMappingURL=astralArrow.js.map