define(["require", "exports", "../characterAction", "../../../utils/engineUtil", "../../../utils/mathUtil", "../../../configs/spritesheetConfiguration", "../../../configs/skillConfiguration", "../../../configs/buffConfiguration", "../buff/baseBuff", "../../../configs/damageCalculationConfiguration", "../projectile/projectileFactory", "../../../configs/projectileConfiguration"], function (require, exports, characterAction_1, engineUtil_1, mathUtil_1, spritesheetConfiguration_1, skillConfiguration_1, buffConfiguration_1, baseBuff_1, damageCalculationConfiguration_1, projectileFactory_1, projectileConfiguration_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class BaseSkill extends characterAction_1.CharacterAction {
        constructor(field, model, npc) {
            super(field);
            this._state = "res";
            this._rushing = false;
            this._model = model;
            this._npc = npc;
            const bf = field.getCurrentScene();
            const skillConfig = bf.configUtil.getConfig(skillConfiguration_1.SkillConfiguration);
            this._dmgConfigUtil = bf.configUtil.getConfig(damageCalculationConfiguration_1.DamageCalculationConfiguration);
            this._buffConfigUtil = bf.configUtil.getConfig(buffConfiguration_1.BuffConfiguration);
            this._config = skillConfig.getConfig(model.configId);
        }
        get rushing() {
            return this._rushing;
        }
        set rushing(value) {
            this._rushing = value;
        }
        get effectKey() {
            return BaseSkill.ResPrefix + this._model.configId;
        }
        get npc() {
            return this._npc;
        }
        get buffConfigUtil() {
            return this._buffConfigUtil;
        }
        get config() {
            return this._config;
        }
        updateFrame() {
            if (this._state == "res") {
                this._state = "init";
                this.preloadRes();
                this.game.load.onLoadComplete.addOnce(this.init, this);
                engineUtil_1.EngineUtil.loadStart(this.game);
            }
            else if (this._state == "done") {
                this.doUpdateFrame();
            }
        }
        preloadRes() {
            const f = this.field;
            const bf = f.getCurrentScene();
            const spritesheetConfig = bf.configUtil.getConfig(spritesheetConfiguration_1.SpritesheetConfiguration);
            const conf = this._config;
            if (conf.pathId) {
                const spritesheet = spritesheetConfig.getConfig(conf.pathId);
                f.checkAndLoadSpriteSheet(this.effectKey, spritesheet.path, spritesheet.dimension[0], spritesheet.dimension[1]);
            }
        }
        init() {
            this._state = "done";
        }
        findTargets() {
            const ret = new Array();
            const monsters = this.field.monsters;
            const pos = new Phaser.Point(this._npc.origPosition.x - this._npc.width / 2, this._npc.origPosition.y);
            for (let m of monsters) {
                if (m.x - m.width / 2 >= pos.x) {
                    continue;
                }
                ret.push(m);
            }
            switch (this._config.preferTarget) {
                case "fastest": {
                    return ret.sort(((a, b) => {
                        return b.moveSpeed - a.moveSpeed;
                    }));
                }
                case "highestDefense": {
                    return ret.sort(((a, b) => {
                        return b.defense - a.defense;
                    }));
                }
                default: {
                    return ret.sort(((a, b) => {
                        return b.x - a.x;
                    }));
                }
            }
        }
        showTargetEffect(target) {
            if (this._config.pathId > 0) {
                let effect = this.game.make.sprite(target.x, target.y, this.effectKey);
                effect.anchor.set(0.5, 0.5);
                effect.animations.add("attack", this._config.animation);
                effect.animations.play("attack", 15, false, true);
                this.field.addChild(effect);
            }
        }
        showProjectileEffect(target, damage) {
            let ret = null;
            if (this._config.projectileId > 0) {
                const scene = this.field.getCurrentScene();
                const proj = scene.serviceProvider.getService(projectileFactory_1.ProjectileFactory);
                const configEntry = scene.configUtil.getConfig(projectileConfiguration_1.ProjectileConfiguration).getConfig(this._config.projectileId);
                ret = proj.create(this.field, configEntry, target, this.npc, damage);
                this.field.addAction(ret);
            }
            return ret;
        }
        showActorEffect(target, speed = 1, times = 1) {
            this._npc.fight(speed, times);
            this._npc.position.set(target.x + this._npc.width, target.y);
        }
        applyDamage(target, damages, buff) {
            if (damages) {
                target.suffer(damages);
            }
            if (buff) {
                target.addBuff(buff, true);
            }
            else if (this.config.buffs) {
                for (let buffId of this.config.buffs) {
                    let b = this.createBuff(buffId);
                    target.addBuff(b, true);
                }
            }
        }
        createBuff(buffId) {
            const buffConfig = this.buffConfigUtil.getConfig(buffId);
            let b = new baseBuff_1.BaseBuff(buffConfig.removeEvent, buffConfig.type, buffConfig.overlap, buffConfig.positive);
            b.duration = buffConfig.duration;
            b.times = buffConfig.times;
            for (let i = 0; i < buffConfig.changePercent.length; i++) {
                b.addEffect(buffConfig.changeStatus[i], buffConfig.changePercent[i], buffConfig.changeValue[i]);
            }
            return b;
        }
        calculateAttack() {
            const c = this._npc;
            const n = c.model;
            let diceFaces;
            if (n.diceFaces < 1) {
                diceFaces = Math.ceil(c.attack * n.diceFaces);
            }
            else {
                diceFaces = n.diceFaces;
            }
            let total = 0;
            for (let i = 0; i < n.diceNum; i++) {
                total += mathUtil_1.MathUtil.getRandomInt(diceFaces) + 1;
            }
            c.checkBuffAfterEvent(1);
            return c.attack + total;
        }
        calculateDamage(target) {
            let total = this.calculateAttack();
            let rand = mathUtil_1.MathUtil.getRandomInt(100);
            let critical = rand < this._npc.model.criticalRate;
            return this.getFinalDamage(total * (critical ? 2 : 1), target, critical);
        }
        getFinalDamage(originDamage, target, critical = false, ignoreDefense = false) {
            if (!ignoreDefense) {
                originDamage *= (1 - target.defense / 100);
            }
            return {
                value: Math.floor(originDamage * this._dmgConfigUtil.getRatio(this._config.atkType, target.defenseType)),
                type: this._config.atkType,
                source: this._npc,
                critical: critical,
                rushing: this._rushing
            };
        }
    }
    BaseSkill.ResPrefix = "skill_effect_";
    exports.BaseSkill = BaseSkill;
});
//# sourceMappingURL=baseSkill.js.map