var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
define(["require", "exports", "./baseScene", "../utils/i18n", "../utils/fontUtil", "../logics/playerLogic", "../models/modelStore", "../ui/windows/husbandMenu", "./battle/battlefield", "../ui/feGameUI", "../ui/windows/monsterMenu", "../utils/stringUtil", "../configs/difficultyConfiguration", "../ui/windows/textMsgBox", "../ui/windows/miscellaneousMenu", "../ui/windows/guideWindow"], function (require, exports, baseScene_1, i18n_1, fontUtil_1, playerLogic_1, modelStore_1, husbandMenu_1, battlefield_1, feGameUI_1, monsterMenu_1, stringUtil_1, difficultyConfiguration_1, textMsgBox_1, miscellaneousMenu_1, guideWindow_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var GsUIName;
    (function (GsUIName) {
        GsUIName["Header"] = "header";
        GsUIName["Difficulty"] = "difficulty";
        GsUIName["Wave"] = "wave";
        GsUIName["Rush"] = "rush";
        GsUIName["Hero"] = "hero";
        GsUIName["Monster"] = "monster";
        GsUIName["Misc"] = "misc";
    })(GsUIName = exports.GsUIName || (exports.GsUIName = {}));
    class GameScene extends baseScene_1.BaseScene {
        constructor(sp) {
            super("game_main", sp);
        }
        preloadRes() {
            super.preloadRes();
            this.checkAndLoadImage(GameScene.ResPrefix + "txtPlate", "common/textplateD.png");
            this.checkAndLoadImage(GameScene.ResPrefix + "txtPlate2", "common/textplateD2.png");
            this.checkAndLoadImage(GameScene.ResPrefix + "field", "home/field/pipo-battlebg001.jpg");
            this.checkAndLoadImage(GameScene.ResPrefix + "header_bg", "common/window/mainwindowA.png");
            this.checkAndLoadImage(GameScene.ResPrefix + "bottom_bg", "common/window/mainwindowD2.png");
        }
        getUI(name) {
            return this.safezone.getByName(name);
        }
        createAsync() {
            return __awaiter(this, void 0, void 0, function* () {
                const sz = this.safezone;
                this._header = this.game.add.group(sz);
                this._header.name = GsUIName.Header;
                this.initHeader();
                let field = this.game.add.image(0, this._header.y + this._header.height, GameScene.ResPrefix + "field", null, sz);
                let field2 = this.game.add.image(field.x + field.width / 2, field.y + field.height * 3 / 2 - 5, GameScene.ResPrefix + "field", null, sz);
                field2.anchor.set(0.5, 0.5);
                field2.scale.set(-1, 1);
                field2.angle = 180;
                const imgY = field.y + field.height * 1.25;
                let img = this.feUI.createPanel(GameScene.ResPrefix + "bottom_bg", 40, this.safeWidth, this.safeHeight - imgY);
                img.position.set(0, field.y + field.height * 1.25);
                sz.add(img);
                let mask = this.game.add.graphics(0, img.y, sz);
                mask.beginFill(0xffffff).drawRect(0, 0, this.safeWidth, this.safeHeight - imgY);
                img.mask = mask;
                this._battleField = new battlefield_1.Battlefield(this, this.safeWidth, img.y - field.y, this._playerModel);
                this._battleField.position.set(field.x, field.y);
                sz.add(this._battleField);
                let graphics = this.game.add.graphics(0, 0, sz);
                graphics.alpha = 0;
                graphics.beginFill(0x000000).drawRect(this._battleField.x, this._battleField.y, this._battleField.width, this._battleField.height);
                this._battleField.mask = graphics;
                this.initButtons();
                let ownedNpc = this.modelStore.loadByType(this._playerModel.id, "npc");
                if (ownedNpc != null) {
                    for (let npc of ownedNpc) {
                        yield this._battleField.addNpc(npc);
                    }
                }
                if (!this._playerModel.passGuide) {
                    this._playerModel.passGuide = true;
                    this.popupWindow(new guideWindow_1.GuideWindow(this));
                }
                if (this._playerModel.hasLinkAccount) {
                    this.platform.loginGoogleAsync().then();
                }
            });
        }
        initHeader() {
            const modelStore = this.serviceProvider.getService(modelStore_1.ModelStore);
            const playerModel = modelStore.loadMe();
            let headerBg = this.feUI.createPanel(GameScene.ResPrefix + "header_bg", 40, this.safeWidth, 100);
            this._header.add(headerBg);
            let loveLbl = this.charUI.getGoldIcon();
            loveLbl.anchor.set(0.5, 0.5);
            loveLbl.position.set(20 + loveLbl.width / 2, loveLbl.height / 2 + 15);
            headerBg.addChild(loveLbl);
            let curLoveLbl = this.game.make.text(0, 0, playerModel.gold.toLocaleString("en"), fontUtil_1.FontUtil.getNumberFont(headerBg.height / 3.5, '#a9fffc'));
            curLoveLbl.anchor.set(0, 0.5);
            curLoveLbl.position.set(loveLbl.x + loveLbl.width / 2 + 5, loveLbl.y + 3);
            headerBg.addChild(curLoveLbl);
            this.addDisposable(playerModel.onPropertyChangedS("gold", this, (data) => {
                curLoveLbl.setText(data.toLocaleString("en"));
            }));
            let lpsLbl = this.charUI.getTimerIcon();
            lpsLbl.anchor.set(0.5, 0.5);
            lpsLbl.position.set(loveLbl.x, loveLbl.y + loveLbl.height + 5);
            headerBg.addChild(lpsLbl);
            let curLpsLbl = this.game.make.text(0, 0, stringUtil_1.StringUtil.convertFromSeconds(playerModel.gaugeInSeconds), fontUtil_1.FontUtil.getNumberFont(headerBg.height / 3.5, '#a9fffc'));
            curLpsLbl.anchor.set(0, 0.5);
            curLpsLbl.position.set(curLoveLbl.x, lpsLbl.y);
            headerBg.addChild(curLpsLbl);
            this.addDisposable(playerModel.onPropertyChangedS("gauge", this, (data) => {
                curLpsLbl.setText(stringUtil_1.StringUtil.convertFromSeconds(playerModel.gaugeInSeconds));
                if (data <= 0 && this._battleField.model.rush) {
                    this.onRushClicked();
                }
            }));
            this._playerModel = playerModel;
        }
        initButtons() {
            const sz = this.safezone;
            let husbandButton = this.feUI.createGameButtonNine("blue", i18n_1.I18n.t("HEROES"), this.onHusbandClicked, this);
            const btnInterval = (this.safeWidth - 3 * husbandButton.width) / 4;
            husbandButton.position.set(btnInterval, this.safeHeight - husbandButton.height - 180);
            husbandButton.setLblPos("center");
            husbandButton.name = GsUIName.Hero;
            sz.add(husbandButton);
            let achButton = this.feUI.createGameButtonNine("yellow", i18n_1.I18n.t("MONSTERS"), this.onMonsterClicked, this);
            achButton.position.set(husbandButton.x + husbandButton.width + btnInterval, husbandButton.y);
            achButton.setLblPos("center");
            achButton.name = GsUIName.Monster;
            sz.add(achButton);
            let historyButton = this.feUI.createGameButtonNine("white", i18n_1.I18n.t("MISC."), this.onMiscClicked, this);
            historyButton.position.set(achButton.x + achButton.width + btnInterval, husbandButton.y);
            historyButton.setLblPos("center");
            historyButton.name = GsUIName.Misc;
            sz.add(historyButton);
            const em = this._battleField.emitter.model;
            const txtPlate = this.feUI.createPanelUnregular(GameScene.ResPrefix + "txtPlate", 10, 20, 10, 20, 250, 54);
            const txtPlate2 = this.feUI.createPanelUnregular(GameScene.ResPrefix + "txtPlate2", 10, 20, 10, 20, 250, 54);
            const difficultyBtn = new feGameUI_1.GameButton(this, txtPlate2, "", this.onDifficultyClicked, this);
            this.changeDifficultyBtnText(difficultyBtn, em);
            difficultyBtn.position.set(husbandButton.x, this._battleField.y + this._battleField.height + 15);
            difficultyBtn.name = GsUIName.Difficulty;
            sz.add(difficultyBtn);
            const waveBtn = new feGameUI_1.GameButton(this, txtPlate, "", this.onWaveClicked, this);
            this.changeWaveBtnText(waveBtn, em);
            waveBtn.position.set(difficultyBtn.x, difficultyBtn.y + difficultyBtn.height + 10);
            waveBtn.name = GsUIName.Wave;
            sz.add(waveBtn);
            em.onPropertyChangedS("level", this, () => {
                this.changeDifficultyBtnText(difficultyBtn, em);
                this.saveAllDbAsync().then();
            });
            em.onPropertyChangedS("percentage", this, () => {
                this.changeWaveBtnText(waveBtn, em);
            });
            em.onPropertyChangedS("configId", this, () => {
                this.changeWaveBtnText(waveBtn, em);
            });
            let rushButton = this.feUI.createSwitchGameButton(this.onRushClicked, this);
            rushButton.scale.set(0.6);
            rushButton.position.set(historyButton.x + historyButton.width - rushButton.width, difficultyBtn.y);
            rushButton.setLblPos("center");
            rushButton.name = GsUIName.Rush;
            sz.add(rushButton);
            this._rushButton = rushButton;
            let timerIcon = this.charUI.getTimerIcon();
            timerIcon.scale.set(1.4);
            timerIcon.anchor.set(1, 0);
            timerIcon.position.set(rushButton.x, rushButton.y);
            sz.add(timerIcon);
            const logic = this.getLogic(playerLogic_1.PlayerLogic);
            this.addDisposable(logic.onEventS("buyNpcSucceed", this, this.onBuyNpc, false));
        }
        changeDifficultyBtnText(btn, em) {
            const difficultyConfigUtil = this.configUtil.getConfig(difficultyConfiguration_1.DifficultyConfiguration);
            const difficultyName = i18n_1.I18n.t(difficultyConfigUtil.get(em.level).name);
            btn.setLblText(difficultyName);
        }
        changeWaveBtnText(btn, em) {
            btn.setLblText(i18n_1.I18n.tn("Wave {0} ({1}%)", em.configId.toString(), (em.percentage * 100).toFixed()));
        }
        onDifficultyClicked() {
            const em = this._battleField.emitter.model;
            const difficultyConfigUtil = this.configUtil.getConfig(difficultyConfiguration_1.DifficultyConfiguration);
            const configEntry = difficultyConfigUtil.get(em.level);
            let msgbox = new textMsgBox_1.TextMsgBox(this, i18n_1.I18n.tn("Monster Level: {0}", configEntry.monsterLevel.toString()) + "\r\n" +
                i18n_1.I18n.tn("Monster Number: {0}%", (configEntry.numMultiplier * 100).toString()));
            this.popupWindow(msgbox);
        }
        onWaveClicked() {
            let msgbox = new textMsgBox_1.TextMsgBox(this, i18n_1.I18n.t("wave.tips"));
            this.popupWindow(msgbox);
        }
        onRushClicked() {
            const bm = this._battleField.model;
            if (bm.rush) {
                this._rushButton.showTex(0);
                bm.rush = false;
            }
            else if (this._playerModel.gauge > 0 && this._battleField.npcs.length > 0) {
                this._rushButton.showTex(1);
                bm.rush = true;
            }
        }
        disableButtons() {
        }
        onMonsterClicked() {
            this.disableButtons();
            let husbandMenu = new monsterMenu_1.MonsterMenu(this);
            this.popupWindow(husbandMenu);
        }
        onMiscClicked() {
            this.disableButtons();
            let miscMenu = new miscellaneousMenu_1.MiscellaneousMenu(this);
            this.popupWindow(miscMenu);
        }
        onHusbandClicked() {
            this.disableButtons();
            let husbandMenu = new husbandMenu_1.HusbandMenu(this);
            this.popupWindow(husbandMenu);
        }
        onBuyNpc(npc) {
            this._battleField.addNpc(npc).then();
        }
    }
    GameScene.ResPrefix = "main_";
    exports.GameScene = GameScene;
});
//# sourceMappingURL=gameScene.js.map