var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
define(["require", "exports", "../../utils/engineUtil", "./monsterCharacter", "./npcCharacter", "../../configs/spritesheetConfiguration"], function (require, exports, engineUtil_1, monsterCharacter_1, npcCharacter_1, spritesheetConfiguration_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class CharacterFactory {
        setGame(value) {
            this._g = value;
            return this;
        }
        setConfigUtil(value) {
            this._c = value;
            return this;
        }
        createNpcForUI(x, y, config) {
            return __awaiter(this, void 0, void 0, function* () {
                const spritesheetConfig = this._c.getConfig(spritesheetConfiguration_1.SpritesheetConfiguration);
                const spritesheet = spritesheetConfig.getConfig(config.pathId);
                return this.createNpc(x, y, spritesheet.path, spritesheet.dimension, config);
            });
        }
        createMonsterForUI(x, y, config) {
            return __awaiter(this, void 0, void 0, function* () {
                const spritesheetConfig = this._c.getConfig(spritesheetConfiguration_1.SpritesheetConfiguration);
                const spritesheet = spritesheetConfig.getConfig(config.pathId);
                return this.createMonster(x, y, spritesheet.path, spritesheet.dimension, config);
            });
        }
        createNpc(x, y, path, dimension, config) {
            return __awaiter(this, void 0, void 0, function* () {
                const game = this._g;
                engineUtil_1.EngineUtil.checkAndLoadSpriteSheet(game, path, path, dimension[0], dimension[1], 0, 0);
                yield engineUtil_1.EngineUtil.loadStartAsync(game);
                const ret = new npcCharacter_1.NpcCharacter(game, x, y, path);
                ret.animations.add("standby", config.standbyAni, 3, true);
                ret.animations.add("fight", config.fightAni);
                return ret;
            });
        }
        createMonster(x, y, path, dimension, config) {
            return __awaiter(this, void 0, void 0, function* () {
                const game = this._g;
                engineUtil_1.EngineUtil.checkAndLoadSpriteSheet(game, path, path, dimension[0] - dimension[3], dimension[1] - dimension[3], dimension[2], dimension[3]);
                yield engineUtil_1.EngineUtil.loadStartAsync(game);
                const ret = new monsterCharacter_1.MonsterCharacter(game, x, y, path);
                ret.animations.add("walk_right", config.animation, 3 * Math.ceil(config.moveSpeed / 32), true);
                ret.smoothed = false;
                return ret;
            });
        }
    }
    exports.CharacterFactory = CharacterFactory;
});
//# sourceMappingURL=characterFactory.js.map