define(["require", "exports", "./baseCharacter", "../../configs/battleFieldConfiguration", "../../utils/fontUtil", "../../scenes/battle/buff/baseBuff"], function (require, exports, baseCharacter_1, battleFieldConfiguration_1, fontUtil_1, baseBuff_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class MonsterCharacter extends baseCharacter_1.BaseCharacter {
        constructor() {
            super(...arguments);
            this._xUnitToPixel = 65536;
            this._yUnitToPixel = 65536;
            this._xUnit = 0;
            this._yUnit = -1;
            this._widthUnit = -1;
            this._damageTweenDelay = 0;
            this._stunBack = 0;
        }
        get moveSpeed() {
            if (this._moveSpeed == undefined) {
                this.updateStatus();
            }
            return this._moveSpeed;
        }
        get model() {
            return this._model;
        }
        get defenseType() {
            return this._model.defenseType;
        }
        get defense() {
            if (this._defense == undefined) {
                this.updateStatus();
            }
            return this._defense;
        }
        get xUnit() {
            return this._xUnit;
        }
        get widthUnit() {
            if (this._widthUnit < 0) {
                this._widthUnit = Math.floor(this.width / this._xUnitToPixel);
            }
            return this._widthUnit;
        }
        get yUnit() {
            if (this._yUnit < 0) {
                this._yUnit = Math.floor(this.y / this._yUnitToPixel);
            }
            return this._yUnit;
        }
        set yUnitToPixel(value) {
            this._yUnitToPixel = value;
        }
        set xUnitToPixel(value) {
            this._xUnitToPixel = value;
        }
        set model(value) {
            this._model = value;
        }
        initBubbles() {
            if (this.model.isBoss) {
                this.addBubble("isBoss", null);
            }
        }
        updateStatus() {
            const buffs = this.buffs;
            const m = this._model;
            this._moveSpeed = m.moveSpeed;
            this._defense = m.defense;
            for (let buff of buffs) {
                for (let c of buff.effects) {
                    switch (c.status) {
                        case baseBuff_1.BaseBuff.MoveSpeedStatus:
                            this._moveSpeed = Math.ceil(this._moveSpeed * Math.pow(1 + c.percent, buff.overlap));
                            break;
                        case baseBuff_1.BaseBuff.DefenseStatus:
                            this._defense = Math.ceil(this._defense * Math.pow(1 + c.percent, buff.overlap));
                            break;
                    }
                }
            }
        }
        updateFrame() {
            super.updateFrame();
            if (!this._model.alive) {
                return;
            }
            this.checkWalk();
            this.checkAlive();
        }
        notifyPropertyChangeS(event, eventData) {
            super.notifyPropertyChangeS(event, eventData);
        }
        onPropertyChangedS(event, caller, listener) {
            return super.onPropertyChangedS(event, caller, listener);
        }
        showDamage(dmg) {
            let fontColor;
            if (dmg.value > 0) {
                fontColor = dmg.critical ? "#ff0000" : "#ffffff";
            }
            else {
                fontColor = "#00ff00";
            }
            const fontSize = 24;
            const duration = 1000;
            const dmgTxt = this.game.make.text(0, 0, Math.abs(dmg.value).toString(), fontUtil_1.FontUtil.getNumberFont(fontSize, fontColor, 'normal', 1));
            dmgTxt.position.set(this.x - this.width / 2, this.y - this.height / 2);
            dmgTxt.sendToBack();
            dmgTxt.alpha = 0;
            this.parent.addChild(dmgTxt);
            let tween = this.game.add.tween(dmgTxt).to({ y: dmgTxt.y - fontSize * 2 }, duration).delay(this._damageTweenDelay);
            let fade = this.game.add.tween(dmgTxt).to({ alpha: 1 }, duration).delay(this._damageTweenDelay);
            this._damageTweenDelay += duration / 3;
            fade.onComplete.addOnce(() => {
                dmgTxt.pendingDestroy = true;
            });
            tween.start();
            fade.start();
        }
        suffer(dmgs) {
            this._damageTweenDelay = 0;
            const m = this._model;
            const maxHp = m.maxHp;
            let totalDamage = 0;
            for (let dmg of dmgs) {
                this.showDamage(dmg);
                if (m.hp > 0) {
                    this._model.hp -= dmg.value;
                    totalDamage += dmg.value;
                    this.notifyPropertyChangeS("damaged", dmg);
                    if (this._model.hp <= 0) {
                        this._model.alive = false;
                        this.pendingDestroy = true;
                        this.notifyPropertyChangeS("destroyed", "attack");
                    }
                }
            }
            if (totalDamage / maxHp >= 0.1) {
                this._stunBack = 3;
            }
            this.playerLogic.handleEvent("damage_to_monster", totalDamage);
        }
        checkAlive() {
            if (this._xUnit > battleFieldConfiguration_1.BattleFieldConfiguration.GridSize) {
                this._model.alive = false;
                this.pendingDestroy = true;
                this.notifyPropertyChangeS("destroyed", "outbound");
            }
        }
        checkWalk() {
            let unitToMove = this.moveSpeed / battleFieldConfiguration_1.BattleFieldConfiguration.FramePerSecond;
            if (this._stunBack > 0) {
                this._stunBack -= 1;
                unitToMove = Math.max(-1, 0 - unitToMove);
            }
            const pixel = unitToMove * this._xUnitToPixel;
            switch (this.direction) {
                case "up":
                    this.y -= pixel;
                    break;
                case "down":
                    this.y += pixel;
                    break;
                case "left":
                    this.x -= pixel;
                    break;
                case "right":
                    this.x += pixel;
                    break;
            }
            this._xUnit += unitToMove;
        }
    }
    exports.MonsterCharacter = MonsterCharacter;
});
//# sourceMappingURL=monsterCharacter.js.map