define(["require", "exports", "./baseCharacter", "../../scenes/battle/skills/skillFactory", "../../utils/configUtil", "../../configs/skillConfiguration", "../../configs/battleFieldConfiguration", "../../utils/mathUtil", "../../scenes/battle/buff/baseBuff"], function (require, exports, baseCharacter_1, skillFactory_1, configUtil_1, skillConfiguration_1, battleFieldConfiguration_1, mathUtil_1, baseBuff_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class ActionResult {
        constructor() {
            this.action = null;
        }
    }
    class NpcCharacter extends baseCharacter_1.BaseCharacter {
        constructor() {
            super(...arguments);
            this._cooldown = 0;
            this._passiveSkills = [];
        }
        get attack() {
            if (this._attack == undefined) {
                this.updateStatus();
            }
            return this._attack;
        }
        get speed() {
            if (this._speed == undefined) {
                this._speed = this._model.speed / battleFieldConfiguration_1.BattleFieldConfiguration.FrameInterval;
            }
            return this._speed;
        }
        get model() {
            return this._model;
        }
        get origPosition() {
            return this._origPosition;
        }
        set origPosition(value) {
            this._origPosition = value;
            this.position.set(value.x, value.y);
        }
        set model(value) {
            if (value != this._model) {
                this._model = value;
                this._model.onPropertyChangedS("level", this, () => {
                    this.updateStatus();
                });
            }
        }
        initialize() {
            super.initialize();
            const skills = this._model.skills;
            const configUtil = this.serviceProvider.getService(configUtil_1.ConfigUtil);
            const skillConfig = configUtil.getConfig(skillConfiguration_1.SkillConfiguration);
            for (let skill of skills) {
                const config = skillConfig.getConfig(skill.configId);
                if (config.passive == 1) {
                    this._passiveSkills.push(this.createSkill(skill, { rushing: false }));
                }
            }
        }
        initBubbles() {
        }
        updateStatus() {
            const buffs = this.buffs;
            const m = this._model;
            let attackPercent = 0, attackValue = 0;
            this._attack = m.attack;
            for (let buff of buffs) {
                for (let c of buff.effects) {
                    switch (c.status) {
                        case baseBuff_1.BaseBuff.AttackStatus:
                            attackPercent += c.percent * buff.overlap;
                            attackValue += c.value * buff.overlap;
                            break;
                    }
                }
            }
            this._attack = Math.ceil((m.attack + attackValue) * (1 + attackPercent));
        }
        updateFrame() {
            super.updateFrame();
            for (let skill of this._passiveSkills) {
                skill.updateFrame();
            }
            this._cooldown++;
        }
        decideAction(model) {
            const ret = new ActionResult();
            if (this._action != null && !this._action.finished) {
                return ret;
            }
            if (this._cooldown < this.speed) {
                return ret;
            }
            this._cooldown = 0;
            let context = { rushing: model.rush && this.field.playerModel.gauge > 0 };
            let skillModels = this.getAvailableSkills(context.rushing ? 2 : 1);
            let rand = mathUtil_1.MathUtil.getRandomInt(100);
            let skill;
            if (rand < this.model.skillRate) {
                skill = skillModels[1];
            }
            else {
                skill = skillModels[0];
            }
            if (skill == null) {
                return ret;
            }
            ret.action = this.createSkill(skill, context);
            return ret;
        }
        getAvailableSkills(condition) {
            const ret = new Array(2);
            const skillModels = this._model.skills;
            for (let model of skillModels) {
                if (model.condition % 100 == condition) {
                    ret.push(model);
                }
            }
            ret.sort((a, b) => {
                return a.condition - b.condition;
            });
            return ret;
        }
        createSkill(model, context) {
            const fact = this.serviceProvider.getService(skillFactory_1.SkillFactory);
            const configUtil = this.serviceProvider.getService(configUtil_1.ConfigUtil);
            const skillConfig = configUtil.getConfig(skillConfiguration_1.SkillConfiguration).getConfig(model.configId);
            const skill = fact.create(skillConfig.type, this.field, model, this);
            skill.rushing = context.rushing;
            return skill;
        }
        suffer(dmgs) {
        }
    }
    exports.NpcCharacter = NpcCharacter;
});
//# sourceMappingURL=npcCharacter.js.map