define(["require", "exports", "./nineSlice", "../baseGameGroup", "../utils/fontUtil"], function (require, exports, nineSlice_1, baseGameGroup_1, fontUtil_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class FrameBaseAnimation extends baseGameGroup_1.BaseGameGroup {
        constructor(scene, imageNames, interval, loop) {
            super(scene);
            for (let name of imageNames) {
                let img = new Phaser.Image(scene.game, 0, 0, name);
                img.visible = false;
                this.addChild(img);
                this.w = img.width;
                this.h = img.height;
            }
            this.idx = 0;
            this.interval = interval;
            this.loop = loop;
            this.started = false;
        }
        start() {
            if (!this.started) {
                this.started = true;
                this.idx = 0;
                this.run();
            }
        }
        stop() {
            if (this.started) {
                this.started = false;
            }
        }
        run() {
            if (this.started) {
                let img = this.getChildAt(this.idx);
                let tween = this.game.add.tween(img).from({ visible: true }, this.interval);
                tween.onComplete.addOnce(this.animeFinished, this);
                tween.start();
            }
        }
        animeFinished() {
            this.idx = this.idx + 1;
            if (this.idx == this.length) {
                if (this.loop) {
                    this.idx = 0;
                }
                else {
                    return;
                }
            }
            this.run();
        }
    }
    exports.FrameBaseAnimation = FrameBaseAnimation;
    class GameButton extends baseGameGroup_1.BaseGameGroup {
        constructor(scene, tex, text, callback, callbackContext) {
            super(scene);
            this.tex = tex;
            this.texArray = [tex];
            if (tex instanceof Phaser.Sprite) {
                tex.inputEnabled = true;
                tex.events.onInputDown.add(this.onDown, this);
                tex.events.onInputUp.add(this.onUp, this);
            }
            else {
                for (let child of tex.children) {
                    if (child instanceof Phaser.Image || child instanceof Phaser.Sprite) {
                        child.events.onInputDown.add(this.onDown, this);
                        child.events.onInputUp.add(this.onUp, this);
                    }
                }
            }
            this.add(tex);
            if (text != null) {
                let lbl;
                if (typeof text === "string") {
                    lbl = this.game.make.text(0, 0, text, fontUtil_1.FontUtil.getTextFont(22, '#ffffff'));
                }
                else {
                    lbl = text;
                }
                this.lbl = lbl;
                this.add(this.lbl);
                this.setLblPos("center");
            }
            this.callback = callback;
            this.callbackContext = callbackContext;
        }
        addIcon(icon) {
            if (this.icon != null) {
                this.remove(this.icon);
            }
            let x, y;
            if (this.lbl == null) {
                x = (this.tex.width - icon.width) / 2;
            }
            else {
                x = this.lbl.x - icon.width - 5;
            }
            y = (this.tex.height - icon.height) / 2;
            icon.position.set(x, y);
            this.add(icon);
            this.icon = icon;
        }
        addTex(tex) {
            tex.ignoreChildInput = true;
            this.texArray.push(tex);
            this.add(tex);
            this.sendToBack(tex);
        }
        showTex(index) {
            this.tex = this.texArray[index];
            this.bringToTop(this.tex);
            if (this.lbl != null) {
                this.bringToTop(this.lbl);
            }
        }
        setLblText(str) {
            if (this.lbl != null) {
                this.lbl.setText(str);
                if (this.lblPos != null) {
                    this.setLblPos(this.lblPos);
                }
            }
        }
        setLblPos(pos) {
            if (this.lbl == null) {
                return;
            }
            if (pos == "center") {
                this.lbl.position.set((this.tex.width - this.lbl.width) / 2, (this.tex.height - this.lbl.height) / 2);
            }
            else if (pos == "bottom") {
                this.lbl.position.set((this.tex.width - this.lbl.width) / 2, this.tex.height - this.lbl.height);
            }
            this.lblPos = pos;
        }
        onUp(object, pointer, isOver) {
            this.game.input.deleteMoveCallback(this.onMove, this);
            if (this.tex instanceof Phaser.Sprite) {
                this.tex.tint = 0xffffff;
            }
            else {
                this.tex.setAll('tint', 0xffffff);
            }
            if (isOver && this.callback != undefined) {
                this.callback.apply(this.callbackContext);
            }
        }
        onDown() {
            if (this.tex instanceof Phaser.Sprite) {
                this.tex.tint = 0xaaaaaa;
            }
            else {
                this.tex.setAll('tint', 0xaaaaaa);
            }
            this.game.input.addMoveCallback(this.onMove, this);
        }
        onMove(pointer, x, y) {
            if (this.tex instanceof Phaser.Sprite) {
                this.tex.tint = 0xffffff;
            }
            else {
                this.tex.setAll('tint', 0xffffff);
            }
        }
        destroy(destroyChildren, soft) {
            this.game.input.deleteMoveCallback(this.onMove, this);
            super.destroy(destroyChildren, soft);
        }
    }
    exports.GameButton = GameButton;
    class FeButtonDecide extends Phaser.Group {
        constructor(base, down, width, height, game, text, callback, callbackContext) {
            super(game, null);
            this.base = base;
            this.down = down;
            this.w = width;
            this.h = height;
            this.down.onChildInputDown.add(this.onDown, this);
            this.down.onChildInputUp.add(this.onUp, this);
            this.down.alpha = 0;
            this.addChild(base);
            this.addChild(down);
            this.flashing = this.game.add.tween(this.down).to({ alpha: 1.00 }, 1000, Phaser.Easing.Default).yoyo(true).loop(true).start();
            this.textLbl = game.make.text(width / 2, height / 2, text, { fontSize: 40, fill: '#FFFFFF' });
            this.textLbl.strokeThickness = 5;
            this.textLbl.anchor.setTo(0.5, 0.5);
            this.add(this.textLbl);
            this.callback = callback;
            this.callbackContext = callbackContext;
        }
        stopAnimation() {
            this.flashing.stop();
        }
        static tweenNode(g, target, w, h) {
            g.add.tween(target.scale).to({
                x: FeButtonDecide.scaleFactor,
                y: FeButtonDecide.scaleFactor
            }, FeButtonDecide.animationInterval, Phaser.Easing.Default, true);
            g.add.tween(target.position).to({
                x: -w * (FeButtonDecide.scaleFactor - 1) / 2,
                y: -h * (FeButtonDecide.scaleFactor - 1) / 2
            }, FeButtonDecide.animationInterval, Phaser.Easing.Default, true);
        }
        static restoreNode(g, target) {
            g.add.tween(target.scale).to({ x: 1, y: 1 }, FeButtonDecide.animationInterval, Phaser.Easing.Default, true);
            g.add.tween(target.position).to({
                x: 0,
                y: 0,
            }, FeButtonDecide.animationInterval, Phaser.Easing.Default, true);
        }
        onDown() {
            this.flashing.pause();
            this.down.alpha = 1.0;
            FeButtonDecide.tweenNode(this.game, this.base, this.w, this.h);
            FeButtonDecide.tweenNode(this.game, this.down, this.w, this.h);
            this.game.add.tween(this.textLbl.scale).to({
                x: FeButtonDecide.scaleFactor,
                y: FeButtonDecide.scaleFactor
            }, FeButtonDecide.animationInterval, Phaser.Easing.Default, true);
        }
        onUp(sender, pointer, isOver) {
            this.flashing.resume();
            this.down.alpha = 0;
            FeButtonDecide.restoreNode(this.game, this.base);
            FeButtonDecide.restoreNode(this.game, this.down);
            this.game.add.tween(this.textLbl.scale).to({
                x: 1,
                y: 1
            }, FeButtonDecide.animationInterval, Phaser.Easing.Default, true);
            if (isOver && this.callback != undefined) {
                this.callback.apply(this.callbackContext);
            }
        }
    }
    FeButtonDecide.scaleFactor = 1.05;
    FeButtonDecide.animationInterval = 100;
    exports.FeButtonDecide = FeButtonDecide;
    var ButtonNColor;
    (function (ButtonNColor) {
        ButtonNColor["Green"] = "button-green";
        ButtonNColor["Yellow"] = "button-yellow";
        ButtonNColor["Red"] = "button-red";
        ButtonNColor["Gray"] = "button-gray";
    })(ButtonNColor = exports.ButtonNColor || (exports.ButtonNColor = {}));
    class FeGameUI {
        constructor(scene) {
            this.resPrefix = "fg_";
            this.scene = scene;
            this.sliceCreator = new nineSlice_1.NineSlice();
        }
        preloadRes() {
            this.scene.checkAndLoadImage(this.resPrefix + "btn_red", "common/button/buttonA_red.png");
            this.scene.checkAndLoadImage(this.resPrefix + "btn_yellow", "common/button/buttonA_yellow.png");
            this.scene.checkAndLoadImage(this.resPrefix + "btn_green", "common/button/buttonA_green.png");
            this.scene.checkAndLoadImage(this.resPrefix + "btn_white", "common/button/buttonA_white.png");
            this.scene.checkAndLoadImage(this.resPrefix + "btn_blue", "common/button/buttonA_blue.png");
            this.scene.checkAndLoadImage(this.resPrefix + "btn_aqua", "common/button/buttonA_aqua.png");
            this.scene.checkAndLoadImage(this.resPrefix + "switch_on", "common/button/switchA_on.png");
            this.scene.checkAndLoadImage(this.resPrefix + "switch_off", "common/button/switchA_off.png");
            this.scene.checkAndLoadImage(this.resPrefix + "btn_round_yellow", "common/button/buttonB_yellow.png");
            this.scene.checkAndLoadImage(this.resPrefix + "btn_round_green", "common/button/buttonB_green.png");
            this.scene.checkAndLoadImage(this.resPrefix + "btn_round_red", "common/button/buttonB_red.png");
        }
        createGameButtonNine(btnColor, text, callback, callbackContext, scale = 1) {
            let key = this.resPrefix + "btn_" + btnColor;
            let tex = this.sliceCreator.create(this.scene.game, key, key, 180 * scale, 60, 29, 48, 29, 48);
            return new GameButton(this.scene, tex, text, callback, callbackContext);
        }
        createMultiTexGameButtonNine(btnColor, text, callback, callbackContext, scale = 1) {
            let gb = this.createGameButtonNine(btnColor[0], text, callback, callbackContext, scale);
            for (let i = 1; i < btnColor.length; i++) {
                let color = btnColor[i];
                let key = this.resPrefix + "btn_" + color;
                let tex = this.sliceCreator.create(this.scene.game, key, key, 180 * scale, 60, 29, 48, 29, 48);
                gb.addTex(tex);
            }
            return gb;
        }
        createSwitchGameButton(callback, callbackContext) {
            const off = this.scene.game.make.sprite(0, 0, this.resPrefix + "switch_off");
            const on = this.scene.game.make.sprite(0, 0, this.resPrefix + "switch_on");
            const btn = new GameButton(this.scene, off, null, callback, callbackContext);
            btn.addTex(on);
            return btn;
        }
        createRoundGameButtonNine(btnColor, text, callback, callbackContext, scale = 1) {
            let key = this.resPrefix + "btn_round_" + btnColor;
            let tex = this.sliceCreator.create(this.scene.game, key, key, 180 * scale, 60, 29, 48, 29, 48);
            return new GameButton(this.scene, tex, text, callback, callbackContext);
        }
        createButtonS(text, callback, callbackContext) {
            const width = 300, height = 86;
            let btn1 = this.sliceCreator.create(this.scene.game, "button_orange.png", "button_orange.png", width, height, 32, 21, 32, 21);
            let btn2 = this.sliceCreator.create(this.scene.game, "button_orange_p.png", "button_orange_p.png", width, height, 32, 21, 32, 21);
            return new FeButtonDecide(btn1, btn2, width, height, this.scene.game, text, callback, callbackContext);
        }
        createButtonN(text, btnColor = ButtonNColor.Green, callback, callbackContext) {
            const width = 197, height = 75;
            let btn1 = this.sliceCreator.create(this.scene.game, btnColor + ".png", btnColor + ".png", width, height, 28, 35, 28, 35);
            let btn2 = this.sliceCreator.create(this.scene.game, btnColor + ".png", btnColor + ".png", width, height, 28, 35, 28, 35);
            return new FeButtonDecide(btn1, btn2, width, height, this.scene.game, text, callback, callbackContext);
        }
        createPanel(key, margin, width, height) {
            return this.sliceCreator.create(this.scene.game, key, key, width, height, margin, margin, margin, margin);
        }
        createPanelUnregular(key, top, left, bottom, right, width, height) {
            return this.sliceCreator.create(this.scene.game, key, key, width, height, top, left, bottom, right);
        }
    }
    exports.FeGameUI = FeGameUI;
});
//# sourceMappingURL=feGameUI.js.map