define(["require", "exports", "./util", "./listViewCore", "./directionalScroller"], function (require, exports, util_1, listViewCore_1, directionalScroller_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    const defaultOptions = {
        direction: 'y',
        autocull: true,
        momentum: true,
        bouncing: true,
        snapping: false,
        overflow: 100,
        padding: 10,
        searchForClicks: false
    };
    class ListView extends listViewCore_1.ListViewCore {
        constructor(game, parent, bounds, options = {}) {
            super(game, parent, util_1.parseBounds(bounds), Object.assign({}, defaultOptions, options));
            this.scroller = new directionalScroller_1.DirectionalScroller(this.game, this._addMask(bounds), Object.assign({ from: 0, to: 0 }, this.options));
            this.scroller.events.onUpdate.add(o => {
                this._setPosition(o.total);
            });
            this.events.onAdded.add(limit => {
                const to = Math.min(-limit, 0);
                this.scroller.setFromTo(0, to);
                if (this.options.searchForClicks) {
                    this.scroller.registerClickables(this.items);
                }
            });
        }
        destroy() {
            this.scroller.destroy();
            this.scroller = null;
            super.destroy();
        }
        reset() {
            this._setPosition(0);
            this.scroller.reset();
        }
    }
    exports.ListView = ListView;
});
//# sourceMappingURL=listView.js.map