define(["require", "exports", "./util"], function (require, exports, util_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    const defaultOptions = {
        direction: 'y',
        autocull: true,
        padding: 10
    };
    class ListViewCore {
        get options() {
            return this._options;
        }
        get items() {
            return this._items;
        }
        get events() {
            return this._events;
        }
        get game() {
            return this._game;
        }
        get scroller() {
            return this._scroller;
        }
        set scroller(value) {
            this._scroller = value;
        }
        constructor(game, parent, bounds, options = {}) {
            this._game = game;
            this.parent = parent;
            this.bounds = bounds;
            this.o = this._options = Object.assign({}, defaultOptions, options);
            this._items = [];
            if (this.o.direction == 'y') {
                this.p = { xy: 'y', wh: 'height' };
            }
            else {
                this.p = { xy: 'x', wh: 'width' };
            }
            this.grp = this._game.add.group(parent);
            this.grp.position.set(bounds.x, bounds.y);
            this._events = { onAdded: new Phaser.Signal() };
            this.position = 0;
            this.grp.mask = this._addMask(bounds);
        }
        add(child) {
            this._items.push(child);
            let xy = 0;
            if (this.grp.children.length > 0) {
                let lastChild = this.grp.getChildAt(this.grp.children.length - 1);
                xy = lastChild[this.p.xy] + util_1.getWidthOrHeight(lastChild, this.p.wh) + this.o.padding;
            }
            child[this.p.xy] = xy;
            this.grp.addChild(child);
            this.length = xy + child[this.p.wh];
            this._events.onAdded.dispatch(this.length - this.bounds[this.p.wh]);
            return child;
        }
        addMultiple(...children) {
            children.forEach(this.add, this);
        }
        remove(child) {
            this.grp.removeChild(child);
            const index = this._items.indexOf(child);
            if (index == -1)
                return;
            this._items.splice(index, 1);
            return child;
        }
        destroy() {
            this._events.onAdded.dispose();
            this._events = null;
            this.grp.destroy();
            this.grp = null;
            this._game = null;
            this.parent = null;
            this._items = null;
        }
        removeAll() {
            this.grp.removeAll();
            this._items = [];
        }
        cull() {
            for (let i = 0; i < this._items.length; i++) {
                let child = this._items[i];
                child.visible = true;
                if (child[this.p.xy] + util_1.getWidthOrHeight(child, this.p.wh) + this.grp[this.p.xy] < this.bounds[this.p.xy]) {
                    child.visible = false;
                }
                else if (child[this.p.xy] + this.grp[this.p.xy] > this.bounds[this.p.xy] + this.bounds[this.p.wh]) {
                    child.visible = false;
                }
            }
        }
        getPositionByItemIndex(index) {
            return -this._items[index][this.p.xy];
        }
        setPosition(position) {
            this.moveToPosition(position);
        }
        moveToPosition(position) {
            this._scroller.setTo(position);
        }
        moveToItem(index) {
            this._scroller.setTo(this.getPositionByItemIndex(index));
        }
        tweenToPosition(position, duration = 1) {
            this._scroller.tweenTo(duration, position);
        }
        tweenToItem(index, duration = 1) {
            this._scroller.tweenTo(duration, this.getPositionByItemIndex(index));
        }
        _setPosition(position) {
            this.position = position;
            this.grp[this.p.xy] = this.bounds[this.p.xy] + position;
            if (this.o.autocull)
                this.cull();
        }
        _addMask(bounds) {
            let mask = this._game.add.graphics(0, 0, this.parent);
            mask.beginFill(0xff0000).drawRect(bounds.x, bounds.y, bounds.width, bounds.height);
            mask.alpha = 0;
            return mask;
        }
    }
    exports.ListViewCore = ListViewCore;
});
//# sourceMappingURL=listViewCore.js.map