define(["require", "exports", "./config"], function (require, exports, config_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    function parseBounds(bounds) {
        bounds.x = bounds.x ? bounds.x : 0;
        bounds.y = bounds.y ? bounds.y : 0;
        if (bounds.width <= 0) {
            console.warn('PhaserListView: bounds.width <= 0');
        }
        else if (bounds.height <= 0) {
            console.warn('PhaserListView: bounds.height <= 0');
        }
        return bounds;
    }
    exports.parseBounds = parseBounds;
    function getWidthOrHeight(displayObject, widthOrHeight) {
        return (displayObject[`nominal${capitalizeFirstLetter(widthOrHeight)}`] || displayObject[widthOrHeight]);
    }
    exports.getWidthOrHeight = getWidthOrHeight;
    function capitalizeFirstLetter(str) {
        return str.charAt(0).toUpperCase() + str.slice(1);
    }
    exports.capitalizeFirstLetter = capitalizeFirstLetter;
    function findChild(children, predicate, scope = null) {
        if (!children)
            return null;
        for (let i = 0; i < children.length; i++) {
            const child = children[i];
            if (!child)
                continue;
            if (predicate.call(scope, child)) {
                return child;
            }
            const found = findChild(child.children, predicate, scope);
            if (found) {
                return found;
            }
        }
        return null;
    }
    exports.findChild = findChild;
    function detectDrag(pointer) {
        const distanceX = Math.abs(pointer.positionDown.x - pointer.positionUp.x);
        const distanceY = Math.abs(pointer.positionDown.y - pointer.positionUp.y);
        const time = pointer.timeUp - pointer.timeDown;
        return (distanceX > config_1.default.AUTO_DETECT_THRESHOLD || distanceY > config_1.default.AUTO_DETECT_THRESHOLD);
    }
    exports.detectDrag = detectDrag;
    function dispatchClicks(pointer, clickables, type) {
        if (type == 'onInputUp' && detectDrag(pointer))
            return;
        const found = findChild(clickables, clickable => {
            let clicked = false;
            if (clickable.input) {
                clicked = clickable.input.checkPointerOver(pointer);
            }
            return clicked;
        });
        if (found &&
            found.events &&
            found.events[type] &&
            found.events[type].dispatch) {
            found.events[type].dispatch(found, pointer, true);
        }
        return found;
    }
    exports.dispatchClicks = dispatchClicks;
});
//# sourceMappingURL=util.js.map