define(["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class NineImage {
        static create(aGame, aWidth, aHeight, aKey, aFrame, aTop = 0, aLeft = 0, aBottom = 0, aRight = 0, aRepeats = false) {
            NineImage._image = aGame.make.image(0, 0, aKey, aFrame);
            NineImage.calculateNineImage(aWidth, aHeight, aTop, aLeft, aBottom, aRight, aRepeats);
            NineImage._nineImage = new Phaser.BitmapData(aGame, "NineImage" + (NineImage._textureKey++), NineImage._width, NineImage._height);
            NineImage.renderNineImage();
            return NineImage._nineImage;
        }
        static calculateNineImage(aWidth, aHeight, aTop, aLeft, aBottom, aRight, aRepeats) {
            let frame = NineImage._image;
            NineImage._centralWidth = frame.width - aLeft - aRight;
            if (NineImage._centralWidth <= 0)
                NineImage._centralWidth = 1;
            NineImage._centralHeight = frame.height - aTop - aBottom;
            if (NineImage._centralHeight <= 0)
                NineImage._centralHeight = 1;
            if (aRepeats) {
                NineImage._horizontalRepeats = aWidth;
                NineImage._verticalRepeats = aHeight;
                NineImage._width = aLeft + aRight + NineImage._centralWidth * aWidth;
                NineImage._height = aTop + aBottom + NineImage._centralHeight * aHeight;
                NineImage._lastWidth = 0;
                NineImage._lastHeight = 0;
            }
            else {
                let w = aWidth - aLeft - aRight;
                NineImage._horizontalRepeats = Math.floor(w / NineImage._centralWidth);
                NineImage._lastWidth = w % NineImage._centralWidth;
                let h = aHeight - aTop - aBottom;
                NineImage._verticalRepeats = Math.floor(h / NineImage._centralHeight);
                NineImage._lastHeight = h % NineImage._centralHeight;
                NineImage._width = aWidth;
                NineImage._height = aHeight;
            }
            NineImage._leftWidth = aLeft;
            NineImage._rightWidth = aRight;
            NineImage._topHeight = aTop;
            NineImage._bottomHeight = aBottom;
        }
        static renderNineImage() {
            let sourceY = 0;
            let destY = 0;
            if (NineImage._topHeight > 0) {
                NineImage.renderNineImageRow(NineImage._image, sourceY, destY, NineImage._topHeight);
                sourceY += NineImage._topHeight;
                destY += NineImage._topHeight;
            }
            for (let i = 0; i < NineImage._verticalRepeats; i++) {
                NineImage.renderNineImageRow(NineImage._image, sourceY, destY, NineImage._centralHeight);
                destY += NineImage._centralHeight;
            }
            if (NineImage._lastHeight > 0) {
                NineImage.renderNineImageRow(NineImage._image, sourceY, destY, NineImage._lastHeight);
                destY += NineImage._lastHeight;
            }
            sourceY += NineImage._centralHeight;
            if (NineImage._bottomHeight > 0) {
                NineImage.renderNineImageRow(NineImage._image, sourceY, destY, NineImage._bottomHeight);
            }
        }
        static renderNineImageRow(aImage, aSourceY, aDestY, aHeight) {
            let sourceX = 0;
            let destX = 0;
            if (NineImage._leftWidth > 0) {
                NineImage._nineImage.copy(aImage, sourceX, aSourceY, NineImage._leftWidth, aHeight, destX, aDestY);
                destX += NineImage._leftWidth;
                sourceX += NineImage._leftWidth;
            }
            for (let i = 0; i < NineImage._horizontalRepeats; i++) {
                NineImage._nineImage.copy(aImage, sourceX, aSourceY, NineImage._centralWidth, aHeight, destX, aDestY);
                destX += NineImage._centralWidth;
            }
            if (NineImage._lastWidth > 0) {
                NineImage._nineImage.copy(aImage, sourceX, aSourceY, NineImage._lastWidth, aHeight, destX, aDestY);
                destX += NineImage._lastWidth;
            }
            sourceX += NineImage._centralWidth;
            if (NineImage._rightWidth > 0) {
                NineImage._nineImage.copy(aImage, sourceX, aSourceY, NineImage._rightWidth, aHeight, destX, aDestY);
            }
        }
    }
    NineImage._textureKey = 0;
    exports.NineImage = NineImage;
});
//# sourceMappingURL=nineImage.js.map