define(["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class NineSlice {
        create(game, key, name, width, height, aTop, aLeft, aBottom, aRight) {
            let frame = null;
            if (name != null) {
                game.cache.getFrameByName(key, name);
            }
            this.baseTexture = game.cache.getBaseTexture(key);
            this.baseFrame = frame == null ? new Phaser.Rectangle(0, 0, this.baseTexture.width, this.baseTexture.height) : frame.getRect();
            this.localWidth = width;
            this.localHeight = height;
            this.topSize = aTop;
            this.leftSize = aLeft;
            this.rightSize = aRight;
            this.bottomSize = aBottom;
            let sizeW = [this.leftSize, this.baseFrame.width - this.rightSize - this.leftSize, this.rightSize];
            let sizeH = [this.topSize, this.baseFrame.height - this.bottomSize - this.topSize, this.bottomSize];
            let textureXs;
            let textureYs;
            if (frame != null && frame.rotated) {
                textureXs = [this.baseFrame.height - this.topSize, this.bottomSize, 0];
                textureYs = [0, this.leftSize, this.baseFrame.width - this.rightSize];
            }
            else {
                textureXs = [0, this.leftSize, this.baseFrame.width - this.rightSize];
                textureYs = [0, this.topSize, this.baseFrame.height - this.bottomSize];
            }
            let finalXs = [0, this.leftSize, this.localWidth - this.rightSize, this.localWidth];
            let finalYs = [0, this.topSize, this.localHeight - this.bottomSize, this.localHeight];
            let group = new Phaser.Group(game, null);
            for (let yi = 0; yi < 3; yi++) {
                for (let xi = 0; xi < 3; xi++) {
                    let s;
                    if (frame != null && frame.rotated) {
                        s = this.createTexturePart(game, textureXs[yi], textureYs[xi], sizeH[yi], sizeW[xi], finalXs[xi], finalYs[yi]);
                    }
                    else {
                        s = this.createTexturePart(game, textureXs[xi], textureYs[yi], sizeW[xi], sizeH[yi], finalXs[xi], finalYs[yi]);
                    }
                    if (s != null) {
                        s.inputEnabled = true;
                        if (frame != null && frame.rotated) {
                            s.height = finalXs[xi + 1] - finalXs[xi];
                            s.width = finalYs[yi + 1] - finalYs[yi];
                            s.angle = -90;
                            s.anchor.setTo(1, 0);
                        }
                        else {
                            s.width = finalXs[xi + 1] - finalXs[xi];
                            s.height = finalYs[yi + 1] - finalYs[yi];
                        }
                        group.addChild(s);
                    }
                }
            }
            return group;
        }
        createTexturePart(game, x, y, width, height, newX, newY) {
            if (width == 0 || height == 0) {
                return null;
            }
            let frame = new PIXI.Rectangle(this.baseFrame.x + x, this.baseFrame.y + y, Math.max(width, 1), Math.max(height, 1));
            let tex = new PIXI.Texture(this.baseTexture, frame);
            return new Phaser.Image(game, newX, newY, tex);
        }
    }
    exports.NineSlice = NineSlice;
});
//# sourceMappingURL=nineSlice.js.map