var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
define(["require", "exports", "../../baseGameGroup", "../../utils/fontUtil", "../listview/listView", "../../utils/i18n", "./husbandMenu", "../../configs/achievementConfiguration"], function (require, exports, baseGameGroup_1, fontUtil_1, listView_1, i18n_1, husbandMenu_1, achievementConfiguration_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class AchievementMenuItem extends baseGameGroup_1.BaseGameGroup {
        constructor(scene, achievementConfig, achievementModel) {
            super(scene);
            this.achievementConfig = achievementConfig;
            this.achievementModel = achievementModel;
        }
        isCompleted() {
            if (this.achievementModel == null) {
                return false;
            }
            return this.achievementModel.progress >= this.achievementConfig.times;
        }
        preloadResAsync() {
            return __awaiter(this, void 0, void 0, function* () {
                this.checkAndLoadImage(AchievementMenuItem.ResPrefix + "bg", "common/window/sabwindowC.png");
                this.checkAndLoadImage(AchievementMenuItem.ResPrefix + "txtLine", "common/window/sabwindowB.png");
                yield this.checkAndLoadImageAsync(AchievementMenuItem.ResPrefix + "check", "common/check_green.png");
            });
        }
        addSubLine(txt, parent) {
        }
        initViewAsync() {
            return __awaiter(this, void 0, void 0, function* () {
                this.removeAll(true);
                const scene = this.getCurrentScene();
                let tex = scene.feUI.createPanel(AchievementMenuItem.ResPrefix + "bg", 40, 560, 168);
                this.add(tex);
                let name = this.achievementConfig.name;
                let nameLbl = this.game.add.text(25, 20, i18n_1.I18n.tn(name, this.achievementConfig.times.toString()), fontUtil_1.FontUtil.getTextFont(28, '#ffffff', 'bold'), this);
                let img = this.getCurrentScene().feUI.createPanel(AchievementMenuItem.ResPrefix + "txtLine", 30, tex.width - 30, nameLbl.height + 10);
                img.position.set(nameLbl.x - 10, nameLbl.y - 5);
                this.add(img);
                const attrFontStyle = fontUtil_1.FontUtil.getTextFont(22, '#ffffff', 'bold');
                const attrNumFontStyle = fontUtil_1.FontUtil.getTextFont(22, '#fcff7f', 'bold');
                let progress = this.achievementModel == null ? 0 : this.achievementModel.progress;
                if (progress >= this.achievementConfig.times) {
                    progress = this.achievementConfig.times;
                    let checkImage = this.game.add.image(0, 0, AchievementMenuItem.ResPrefix + "check", null, this);
                    checkImage.position.set(tex.width - checkImage.width, tex.height - checkImage.height);
                }
                let progressLbl = this.game.add.text(nameLbl.x, nameLbl.y + nameLbl.height + 5, `${progress}/${this.achievementConfig.times}`, attrNumFontStyle, this);
                let descLbl = this.game.add.text(progressLbl.x, progressLbl.y + progressLbl.height, i18n_1.I18n.t(this.achievementConfig.description), attrFontStyle, this);
                descLbl.wordWrap = true;
                descLbl.wordWrapWidth = tex.width - 55;
            });
        }
    }
    AchievementMenuItem.ResPrefix = "ami_";
    class AchievementMenu extends husbandMenu_1.HusbandMenu {
        initListViewAsync() {
            return __awaiter(this, void 0, void 0, function* () {
                const scene = this.getCurrentScene();
                const player = scene.modelStore.loadMe();
                const achievements = scene.modelStore.loadByType(player.id, "achievement");
                const achievementConfiguration = scene.configUtil.getConfig(achievementConfiguration_1.AchievementConfiguration);
                let configs = achievementConfiguration.getOrderedList();
                let listView = null;
                const grp = this.game.make.group();
                const arr = new Array();
                let notCompletedIndex = 0;
                let typesToSkip = new Set();
                for (let entry of configs) {
                    if (entry.enable == 0 || typesToSkip.has(entry.type)) {
                        continue;
                    }
                    let model = null;
                    for (let temp of achievements) {
                        if (temp.type == entry.type) {
                            model = temp;
                            break;
                        }
                    }
                    let item = new AchievementMenuItem(scene, entry, model);
                    yield item.preloadResAsync();
                    if (item.isCompleted()) {
                        arr.push(item);
                        scene.platform.updateAchievementsAsync(entry.googlePlayId).then();
                    }
                    else {
                        typesToSkip.add(entry.type);
                        arr.splice(notCompletedIndex, 0, item);
                        notCompletedIndex++;
                    }
                    if (listView == null) {
                        yield item.initViewAsync();
                        const bounds = new Phaser.Rectangle(0, 0, item.width, this.bg.height - this._grayLayerHeight - 40);
                        const options = {
                            direction: 'y',
                            overflow: 100,
                            padding: 10
                        };
                        grp.position.set((this.bg.width - item.width) / 2, this.grayLayer.y + this.grayLayer.height);
                        listView = new listView_1.ListView(this.game, grp, bounds, options);
                    }
                    else {
                        item.initViewAsync().then();
                    }
                }
                for (let entry of arr) {
                    listView.add(entry);
                }
                this.add(grp);
            });
        }
    }
    exports.AchievementMenu = AchievementMenu;
});
//# sourceMappingURL=achievementMenu.js.map