var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
define(["require", "exports", "../nineSlice", "../../utils/fontUtil", "../../baseGameGroup", "../../utils/engineUtil"], function (require, exports, nineSlice_1, fontUtil_1, baseGameGroup_1, engineUtil_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class WindowPage extends baseGameGroup_1.BaseGameGroup {
        constructor() {
            super(...arguments);
            this._initialized = false;
        }
        get initialized() {
            return this._initialized;
        }
        set initialized(value) {
            this._initialized = value;
        }
        get pageWidth() {
            return this._pageWidth;
        }
        set pageWidth(value) {
            this._pageWidth = value;
        }
        get pageHeight() {
            return this._pageHeight;
        }
        set pageHeight(value) {
            this._pageHeight = value;
        }
        onShow() {
        }
    }
    exports.WindowPage = WindowPage;
    class BaseWindow extends baseGameGroup_1.BaseGameGroup {
        constructor(scene) {
            super(scene);
            this._enableBackBtn = true;
        }
        get winHeight() {
            return this._winHeight;
        }
        get winWidth() {
            return this._winWidth;
        }
        set enableBackBtn(value) {
            this._enableBackBtn = value;
        }
        get enableBackBtn() {
            return this._enableBackBtn;
        }
        preloadRes() {
        }
        preloadResAsync() {
            return __awaiter(this, void 0, void 0, function* () {
                this.preloadRes();
                yield engineUtil_1.EngineUtil.loadStartAsync(this.game);
            });
        }
        initView() {
        }
        onShow() {
        }
        onClose() {
        }
        closeMe() {
            this.onClose();
            let scene = this.getCurrentScene();
            scene.closeWindow(this);
        }
    }
    exports.BaseWindow = BaseWindow;
    class DefaultWindow extends BaseWindow {
        get leftUpper() {
            return this._leftUpper;
        }
        get rightUpper() {
            return this._rightUpper;
        }
        get leftBottom() {
            return this._leftBottom;
        }
        preloadRes() {
            this.checkAndLoadImage("win_bottom", "common/window/b.png");
            this.checkAndLoadImage("win_left", "common/window/l.png");
            this.checkAndLoadImage("win_leftbottom", "common/window/lb.png");
            this.checkAndLoadImage("win_leftupper", "common/window/lu.png");
            this.checkAndLoadImage("win_right", "common/window/r.png");
            this.checkAndLoadImage("win_rightbottom", "common/window/rb.png");
            this.checkAndLoadImage("win_rightupper", "common/window/ru.png");
            this.checkAndLoadImage("win_upper", "common/window/u.png");
            this.checkAndLoadImage("win_close", "common/window/close.png");
            this.checkAndLoadImage("win_bg_gray", "common/window/bg-gray.png");
            this.checkAndLoadImage("win_title", "common/window/title.png");
            this.checkAndLoadImage("win_tab1", "common/window/tab1.png");
            this.checkAndLoadImage("win_tab2", "common/window/tab2.png");
        }
        initView() {
            const partScale = 0.5, vloops = 5, hloops = 3;
            this._leftUpper = this.game.add.image(0, 0, "win_leftupper", null, this);
            this._leftUpper.scale.set(partScale);
            let u;
            for (let i = 0; i < hloops; i++) {
                u = this.game.add.image(0, 0, "win_upper", null, this);
                u.scale.set(partScale);
                u.position.set(this._leftUpper.width + u.width * i, 0);
            }
            this._rightUpper = this.game.add.image(u.position.x + u.width, 0, "win_rightupper", null, this);
            this._rightUpper.scale.set(partScale);
            let l;
            for (let i = 0; i < vloops; i++) {
                l = this.game.add.image(0, 0, "win_left", null, this);
                l.scale.set(partScale);
                l.position.set(0, this._leftUpper.height + l.height * i);
            }
            let r;
            for (let i = 0; i < vloops; i++) {
                r = this.game.add.image(0, 0, "win_right", null, this);
                r.scale.set(partScale);
                r.position.set(this._rightUpper.position.x, this._rightUpper.height + r.height * i);
            }
            this._leftBottom = this.game.add.image(0, l.position.y + l.height, "win_leftbottom", null, this);
            this._leftBottom.scale.set(partScale);
            let b;
            for (let i = 0; i < hloops; i++) {
                b = this.game.add.image(0, 0, "win_bottom", null, this);
                b.scale.set(partScale);
                b.position.set(this._leftBottom.width + b.width * i, this._leftBottom.position.y);
            }
            let rb = this.game.add.image(this._rightUpper.position.x, b.position.y, "win_rightbottom", null, this);
            rb.scale.set(partScale);
            let sliceCreator = new nineSlice_1.NineSlice();
            let bg = sliceCreator.create(this.game, "win_bg_gray", "win_bg_gray", this._rightUpper.x + this._rightUpper.width - 2, this._leftBottom.y + this._leftBottom.height - 2, 5, 5, 5, 5);
            bg.position.set(1, 1);
            this.add(bg);
            this.sendToBack(bg);
            this._titleBg = this.game.add.image(this.width / 2, this._rightUpper.y, "win_title", null, this);
            this._titleBg.anchor.set(0.5, 0.5);
            this._titleBg.scale.set(partScale);
            let titleLabel = this.game.make.text(0, 0, this._titleText, fontUtil_1.FontUtil.getTitleFont());
            titleLabel.anchor.set(0.5, 0.5);
            titleLabel.scale.set(1 / partScale);
            this._titleBg.addChild(titleLabel);
            this._winWidth = this.width;
            this._winHeight = this.height;
            let closeBtn = this.game.make.button(0, 0, "win_close", this.onCloseBtnPressed, this);
            closeBtn.anchor.set(0.5, 0.5);
            closeBtn.scale.set(partScale + 0.1);
            closeBtn.position.set(this._rightUpper.x + this._rightUpper.width * 0.55, this._rightUpper.y + this._rightUpper.height * 0.35);
            this.add(closeBtn);
        }
        setTitle(title) {
            this._titleText = title;
        }
        onCloseBtnPressed() {
            this.onClose();
            this.getCurrentScene().closeWindow(this);
        }
    }
    exports.DefaultWindow = DefaultWindow;
});
//# sourceMappingURL=baseWindow.js.map