define(["require", "exports", "./baseWindow", "../../utils/i18n", "../../utils/eventbus"], function (require, exports, baseWindow_1, i18n_1, eventbus_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class CustomMsgBox extends baseWindow_1.BaseWindow {
        constructor(scene, container, winWidth, winHeight) {
            super(scene);
            this.eventBus = new eventbus_1.Eventbus();
            this._showConfirmButton = false;
            this._hideAllButton = false;
            if (winWidth == null) {
                winWidth = CustomMsgBox.DefaultWidth;
            }
            if (winHeight == null) {
                winHeight = CustomMsgBox.DefaultHeight;
            }
            this._container = container;
            this._winWidth = winWidth;
            this._winHeight = winHeight;
        }
        set container(value) {
            this._container = value;
        }
        set showConfirmButton(value) {
            this._showConfirmButton = value;
        }
        set hideAllButton(value) {
            this._hideAllButton = value;
        }
        preloadRes() {
            this.checkAndLoadImage(CustomMsgBox.ResPrefix + "bg", "common/window/mainwindowE2.png");
        }
        initView() {
            const uifact = this.getCurrentScene().feUI;
            let bg = uifact.createPanel(CustomMsgBox.ResPrefix + "bg", 55, this._winWidth, this._winHeight);
            this.add(bg);
            let btnHeight = 0;
            if (!this._hideAllButton) {
                let closeBtn = uifact.createRoundGameButtonNine("red", i18n_1.I18n.t("menu.close"), this.onCloseBtnPressed, this);
                this.add(closeBtn);
                let closeBtnX = (bg.width - closeBtn.width) / 2, closeBtnY = bg.height - closeBtn.height - 15;
                if (this._showConfirmButton) {
                    let okBtn = uifact.createRoundGameButtonNine("green", i18n_1.I18n.t("menu.ok"), this.onOkBtnPressed, this);
                    this.add(okBtn);
                    closeBtnX = (bg.width - closeBtn.width - 20 - okBtn.width) / 2;
                    okBtn.position.set(closeBtnX + closeBtn.width + 20, closeBtnY);
                }
                closeBtn.position.set(closeBtnX, closeBtnY);
                btnHeight = closeBtn.height;
            }
            this._container.position.set((bg.width - this._container.width) / 2, (bg.height - btnHeight - this._container.height) / 2);
            this.add(this._container);
        }
        on(event, caller, listener) {
            return this.eventBus.on(event, (event) => {
                listener.call(caller, event);
            });
        }
        onCloseBtnPressed() {
            this.eventBus.emit("close", this);
            this.closeMe();
        }
        onOkBtnPressed() {
            this.eventBus.emit("ok", this);
            this.closeMe();
        }
    }
    CustomMsgBox.DefaultWidth = 500;
    CustomMsgBox.DefaultHeight = 400;
    CustomMsgBox.ResPrefix = "msgbox_";
    exports.CustomMsgBox = CustomMsgBox;
});
//# sourceMappingURL=customMsgBox.js.map