define(["require", "exports", "./baseWindow", "../../scenes/gameScene", "../../utils/i18n", "./customMsgBox", "../../utils/fontUtil"], function (require, exports, baseWindow_1, gameScene_1, i18n_1, customMsgBox_1, fontUtil_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class GuideWindow extends baseWindow_1.BaseWindow {
        constructor(scene) {
            super(scene);
            this.steps = [];
        }
        preloadRes() {
            super.preloadRes();
            this.checkAndLoadImage(GuideWindow.ResPrefix + "arrow", "common/extra_1.png");
            this.checkAndLoadImage(GuideWindow.ResPrefix + "bg", "common/window/mainwindowE2.png");
        }
        initView() {
            const scene = this.getCurrentScene();
            const placeHolder = this.game.add.graphics(0, 0, this);
            this._winWidth = scene.safeWidth;
            this._winHeight = scene.safeHeight;
            placeHolder.drawRect(0, 0, this.winWidth, this.winHeight);
            this.enableBackBtn = false;
        }
        onShow() {
            super.onShow();
            const scene = this.getCurrentScene();
            this.arrow = this.game.add.image(0, 0, GuideWindow.ResPrefix + "arrow", null, this);
            this.arrow.anchor.set(0.5, 0.5);
            this.arrow.visible = false;
            this.indicator = this.game.add.graphics(0, 0, this);
            const bg = scene.feUI.createPanel(GuideWindow.ResPrefix + "bg", 55, customMsgBox_1.CustomMsgBox.DefaultWidth, customMsgBox_1.CustomMsgBox.DefaultHeight);
            bg.visible = false;
            this.add(bg);
            this.bg = bg;
            const okBtn = scene.feUI.createRoundGameButtonNine("green", i18n_1.I18n.t("menu.ok"), this.onOkBtnPressed, this);
            okBtn.position.set((bg.width - okBtn.width) / 2, bg.height - okBtn.height - 20);
            bg.add(okBtn);
            this.okBtn = okBtn;
            const txt = scene.game.make.text(0, 0, "", fontUtil_1.FontUtil.getTextFont(24, '#000000', fontUtil_1.FontUtil.DFW, 0));
            txt.wordWrap = true;
            txt.wordWrapWidth = bg.width - 24;
            bg.add(txt);
            this.txt = txt;
            this.steps.push({ desc: i18n_1.I18n.t("guide.welcome") });
            const header = scene.getUI(gameScene_1.GsUIName.Header);
            this.steps.push({ rect: new Phaser.Rectangle(header.position.x + 5, header.position.y + 5, header.width - 10, header.height - 10), desc: i18n_1.I18n.t("guide.resource") });
            const difficulty = scene.getUI(gameScene_1.GsUIName.Difficulty);
            const wave = scene.getUI(gameScene_1.GsUIName.Wave);
            this.steps.push({
                rect: new Phaser.Rectangle(difficulty.position.x, difficulty.position.y, difficulty.width, wave.y + wave.height - difficulty.y),
                desc: i18n_1.I18n.t("guide.progress")
            });
            const rush = scene.getUI(gameScene_1.GsUIName.Rush);
            this.steps.push({
                rect: new Phaser.Rectangle(rush.position.x, rush.position.y, rush.width, rush.height),
                desc: i18n_1.I18n.t("guide.rush")
            });
            const hero = scene.getUI(gameScene_1.GsUIName.Hero);
            this.steps.push({
                rect: new Phaser.Rectangle(hero.position.x, hero.position.y, hero.width, hero.height),
                desc: i18n_1.I18n.t("guide.hero")
            });
            const monster = scene.getUI(gameScene_1.GsUIName.Monster);
            this.steps.push({
                rect: new Phaser.Rectangle(monster.position.x, monster.position.y, monster.width, monster.height),
                desc: i18n_1.I18n.t("guide.monster")
            });
            const misc = scene.getUI(gameScene_1.GsUIName.Misc);
            this.steps.push({
                rect: new Phaser.Rectangle(misc.position.x, misc.position.y, misc.width, misc.height),
                desc: i18n_1.I18n.t("guide.misc")
            });
            this.onOkBtnPressed();
        }
        updateGuide(rect, desc) {
            const bgW = this.bg.width, bgH = this.bg.height;
            if (rect == null) {
                this.bg.position.set((this.winWidth - bgW) / 2, (this.winHeight - bgH) / 2);
                this.arrow.visible = false;
            }
            else {
                this.indicator.clear();
                this.indicator.lineStyle(3, 0xff0000, 1);
                this.indicator.drawRoundedRect(rect.x, rect.y, rect.width, rect.height, 9);
                if (rect.y > this.winHeight / 2) {
                    this.arrow.angle = 90;
                    this.arrow.position.set(rect.x + rect.halfWidth, rect.y - this.arrow.height / 2);
                    this.bg.position.set((this.winWidth - bgW) / 2, this.arrow.y - this.arrow.height / 2 - bgH);
                }
                else {
                    this.arrow.angle = -90;
                    this.arrow.position.set(rect.x + rect.halfWidth, rect.y + rect.height + this.arrow.height / 2);
                    this.bg.position.set((this.winWidth - bgW) / 2, this.arrow.y + this.arrow.height / 2);
                }
                this.arrow.visible = true;
            }
            this.txt.setText(desc);
            this.txt.position.set((bgW - this.txt.width) / 2, (this.okBtn.y - this.txt.height) / 2);
            this.bg.visible = true;
        }
        onOkBtnPressed() {
            if (this.steps.length == 0) {
                this.closeMe();
            }
            else {
                const step = this.steps.shift();
                this.updateGuide(step.rect, step.desc);
            }
        }
    }
    GuideWindow.ResPrefix = "gw_";
    exports.GuideWindow = GuideWindow;
});
//# sourceMappingURL=guideWindow.js.map