var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
define(["require", "exports", "../../baseGameGroup", "../../configs/npcConfiguration", "../feGameUI", "./npcDetailWindow", "../../utils/fontUtil", "./baseWindow", "../listview/listView", "../../logics/playerLogic", "../../utils/i18n", "../characters/characterFactory", "../characters/characterUI", "../../configs/skillConfiguration", "../../configs/npcDefaultSkillConfiguration"], function (require, exports, baseGameGroup_1, npcConfiguration_1, feGameUI_1, npcDetailWindow_1, fontUtil_1, baseWindow_1, listView_1, playerLogic_1, i18n_1, characterFactory_1, characterUI_1, skillConfiguration_1, npcDefaultSkillConfiguration_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class HusbandMenuItem extends baseGameGroup_1.BaseGameGroup {
        constructor(scene, npcConfig) {
            super(scene);
            this._npcConfig = npcConfig;
            this._charUI = new characterUI_1.CharacterUI(this.getCurrentScene());
        }
        preloadResAsync() {
            return __awaiter(this, void 0, void 0, function* () {
                this._charUI.preloadResAsync().then();
                yield this.checkAndLoadImageAsync(HusbandMenuItem.ResPrefix + "bg", "common/window/sabwindowC.png");
            });
        }
        initViewAsync() {
            return __awaiter(this, void 0, void 0, function* () {
                this.removeAll(true);
                const scene = this.getCurrentScene();
                const charFact = scene.serviceProvider.getService(characterFactory_1.CharacterFactory);
                const modelStore = scene.modelStore;
                const player = modelStore.loadMe();
                let ownedNpc = modelStore.loadByType(player.id, "npc");
                let model;
                for (let npc of ownedNpc) {
                    if (npc.configId == this._npcConfig.id) {
                        model = npc;
                        break;
                    }
                }
                let name = this._npcConfig.name;
                let tex = scene.feUI.createPanel(HusbandMenuItem.ResPrefix + "bg", 40, 560, 135);
                let bgBtn = new feGameUI_1.GameButton(this.getCurrentScene(), tex, null, () => {
                    let win = new npcDetailWindow_1.NpcDetailWindow(this.getCurrentScene(), player, this._npcConfig, model);
                    scene.popupWindow(win);
                }, this);
                this.add(bgBtn);
                let character = yield charFact.createNpcForUI(0, tex.height - 5, this._npcConfig);
                character.anchor.set(0, 1);
                character.standby();
                this.addChild(character);
                let nameLbl = this.game.add.text(character.x + character.width, 10, i18n_1.I18n.t(name), fontUtil_1.FontUtil.getTextFont(32, '#ffffff', 'bold'), this);
                const defaultSkills = scene.configUtil.getConfig(npcDefaultSkillConfiguration_1.NpcDefaultSkillConfiguration).get(this._npcConfig.id);
                if (defaultSkills != null) {
                    const firstSkill = scene.configUtil.getConfig(skillConfiguration_1.SkillConfiguration).getConfig(defaultSkills[0].skillId);
                    const atkTypeIcon = this._charUI.getAtkTypeIcon(firstSkill.atkType);
                    if (atkTypeIcon != null) {
                        atkTypeIcon.anchor.set(0, 0.5);
                        atkTypeIcon.position.set(nameLbl.x + nameLbl.width + 10, nameLbl.y + nameLbl.height / 2);
                        this.add(atkTypeIcon);
                    }
                }
                const attrFontStyle = fontUtil_1.FontUtil.getTextFont(22, '#ffffff', 'bold');
                const attrNumFontStyle = fontUtil_1.FontUtil.getNumberFont(22, '#fcff7f', 'bold');
                let lv = "?", atk = "?", tap = "?", spd = "?";
                let lvLbl = this.game.add.text(nameLbl.x, nameLbl.y + nameLbl.height, i18n_1.I18n.t("stat.lv"), attrFontStyle, this);
                let lvNumLbl = this.game.add.text(lvLbl.x + lvLbl.width + 10, lvLbl.y, lv, attrNumFontStyle, this);
                let spdLbl = this.game.add.text(lvLbl.x + 230, lvLbl.y, i18n_1.I18n.t("stat.spd"), attrFontStyle, this);
                let spdNumLbl = this.game.add.text(spdLbl.x + spdLbl.width + 10, spdLbl.y, spd, attrNumFontStyle, this);
                let atkLbl = this.game.add.text(lvLbl.x, lvLbl.y + lvLbl.height, i18n_1.I18n.t("stat.atk"), attrFontStyle, this);
                let atkNumLbl = this.game.add.text(atkLbl.x + atkLbl.width + 10, atkLbl.y, atk, attrNumFontStyle, this);
                let tapLbl = this.game.add.text(atkLbl.x + 230, atkLbl.y, i18n_1.I18n.t("stat.cri"), attrFontStyle, this);
                let criNumLbl = this.game.add.text(tapLbl.x + tapLbl.width + 10, atkLbl.y, tap, attrNumFontStyle, this);
                if (model != null) {
                    this.refreshLabels(model, lvNumLbl, atkNumLbl, criNumLbl, spdNumLbl);
                    this.addDisposable(model.onPropertyChangedS("level", this, () => {
                        this.refreshLabels(model, lvNumLbl, atkNumLbl, criNumLbl, spdNumLbl);
                    }));
                }
            });
        }
        refreshLabels(model, lvLbl, atkLbl, criLbl, spdLbl) {
            let lv = model.level.toString();
            let atk = model.minAttack.toString() + " - " + model.maxAttack.toString();
            let tap = model.criticalRate + "%";
            let spd = (1000 / model.speed).toFixed(1);
            lvLbl.setText(lv);
            atkLbl.setText(atk);
            criLbl.setText(tap);
            spdLbl.setText(spd);
        }
    }
    HusbandMenuItem.ResPrefix = "hmi_";
    class HusbandMenu extends baseWindow_1.BaseWindow {
        constructor() {
            super(...arguments);
            this._grayLayerHeight = 60;
            this.resPrefix = "hm_";
        }
        get grayLayer() {
            return this._grayLayer;
        }
        get bg() {
            return this._bg;
        }
        set bg(value) {
            this._bg = value;
        }
        preloadRes() {
            this.checkAndLoadImage(this.resPrefix + "bg", "common/window/mainwindowI.png");
        }
        initView() {
            const scene = this.getCurrentScene();
            this._bg = scene.feUI.createPanel(this.resPrefix + "bg", 40, 640, 800);
            this.add(this._bg);
        }
        onShow() {
            this._grayLayer = this.game.make.graphics(this._bg.width - 50, 5);
            this._grayLayer.beginFill(0x000000);
            this._grayLayer.drawRect(0, 0, 50, this._grayLayerHeight);
            this._grayLayer.endFill();
            this._grayLayer.alpha = 0;
            this._grayLayer.inputEnabled = true;
            this._grayLayer.events.onInputUp.addOnce(this.closeMe, this);
            this.initListViewAsync().then(() => {
                this.addChild(this._grayLayer);
                let closeLbl = this.game.make.text(0, 0, "x", fontUtil_1.FontUtil.getTextFont(48, '#ff0000'));
                closeLbl.position.set(this._bg.width - closeLbl.width - 18, 0);
                this.addChild(closeLbl);
            });
        }
        initListViewAsync() {
            return __awaiter(this, void 0, void 0, function* () {
                const scene = this.getCurrentScene();
                const npcConfig = scene.configUtil.getConfig(npcConfiguration_1.NpcConfiguration);
                let configs = npcConfig.getAll();
                configs.sort((a, b) => {
                    return a.initGold - b.initGold;
                });
                let listView = null;
                const grp = this.game.make.group();
                let i = 0;
                for (let entry of configs) {
                    let item = new HusbandMenuItem(this.getCurrentScene(), entry);
                    yield item.preloadResAsync();
                    if (listView == null) {
                        yield item.initViewAsync();
                        const bounds = new Phaser.Rectangle(0, 0, item.width, this._bg.height - this._grayLayerHeight - 40);
                        const options = {
                            direction: 'y',
                            overflow: 100,
                            padding: 0,
                            searchForClicks: true
                        };
                        grp.position.set((this._bg.width - item.width) / 2, this._grayLayer.y + this._grayLayer.height);
                        listView = new listView_1.ListView(this.game, grp, bounds, options);
                    }
                    else {
                        item.initViewAsync().then();
                    }
                    listView.add(item);
                    i++;
                }
                this.add(grp);
                const logic = scene.getLogic(playerLogic_1.PlayerLogic);
                this.addDisposable(logic.onEventS("buyNpcSucceed", this, this.onBuyNpc));
            });
        }
        onBuyNpc() {
            this.closeMe();
        }
    }
    exports.HusbandMenu = HusbandMenu;
});
//# sourceMappingURL=husbandMenu.js.map