var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
define(["require", "exports", "./husbandMenu", "../../utils/fontUtil", "../../utils/i18n", "./achievementMenu", "../../utils/stringUtil", "../../utils/logger", "./customMsgBox", "../../configs/gameConfiguration", "./textMsgBox", "../../logics/playerLogic"], function (require, exports, husbandMenu_1, fontUtil_1, i18n_1, achievementMenu_1, stringUtil_1, logger_1, customMsgBox_1, gameConfiguration_1, textMsgBox_1, playerLogic_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class MiscellaneousMenu extends husbandMenu_1.HusbandMenu {
        constructor() {
            super(...arguments);
            this.rewardInterval = 20 * 3600;
            this.checkInRemaining = 0;
        }
        preloadRes() {
            this.checkAndLoadImage("common/window/mainwindowE2.png");
            this.checkAndLoadImage("googleSignIn", "common/ic_play_games_badge_green.png");
        }
        initView() {
            const scene = this.getCurrentScene();
            this.bg = scene.feUI.createPanel("common/window/mainwindowE2.png", 55, 480, 480);
            this.add(this.bg);
            this.player = scene.modelStore.loadMe();
        }
        update() {
            this.updateCheckIn();
        }
        updateCheckIn() {
            const remaining = Math.ceil(this.rewardInterval - (Date.now() - this.player.lastGetReward) / 1000);
            if (remaining != this.checkInRemaining) {
                this.checkInRemaining = remaining;
                if (remaining > 0) {
                    this.checkInBtn.setLblText(i18n_1.I18n.tn("Check-in ({0})", stringUtil_1.StringUtil.convertFromSeconds(remaining)));
                }
                else {
                    this.checkInBtn.setLblText(i18n_1.I18n.t("Check-in"));
                }
            }
        }
        initListViewAsync() {
            return __awaiter(this, void 0, void 0, function* () {
                const btnInterval = 30;
                const btnScale = 2;
                const scene = this.getCurrentScene();
                const achievementBtn = scene.feUI.createGameButtonNine("aqua", i18n_1.I18n.t("achievements"), this.openAchievements, this, btnScale);
                achievementBtn.position.set((this.bg.width - achievementBtn.width) / 2, this.grayLayer.y + this.grayLayer.height);
                this.add(achievementBtn);
                const checkInBtn = scene.feUI.createGameButtonNine("aqua", "", this.getDailyReward, this, btnScale);
                checkInBtn.position.set(achievementBtn.x, achievementBtn.y + achievementBtn.height + btnInterval);
                this.add(checkInBtn);
                this.checkInBtn = checkInBtn;
                this.updateCheckIn();
                let accountBtn;
                if (this.player.hasLinkAccount) {
                    accountBtn = scene.feUI.createGameButtonNine("aqua", i18n_1.I18n.t("Unlink account"), this.openUnlinkAccount, this, btnScale);
                }
                else {
                    accountBtn = scene.feUI.createGameButtonNine("aqua", i18n_1.I18n.t("Link Account"), this.openLinkAccount, this, btnScale);
                }
                accountBtn.position.set(achievementBtn.x, checkInBtn.y + checkInBtn.height + btnInterval);
                const signInIcon = this.game.make.image(0, 0, "googleSignIn");
                signInIcon.scale.set(0.75);
                accountBtn.addIcon(signInIcon);
                this.add(accountBtn);
                const creditBtn = scene.feUI.createGameButtonNine("aqua", i18n_1.I18n.t("Credit"), this.openCredit, this, btnScale);
                creditBtn.position.set(achievementBtn.x, accountBtn.y + accountBtn.height + btnInterval);
                this.add(creditBtn);
                const platform = scene.platform;
                const userId = platform.playFabId;
                if (userId != null) {
                    let idLbl = this.game.make.text(0, 0, `(${userId})`, fontUtil_1.FontUtil.getTextFont(20, '#ffffff', "normal"));
                    idLbl.position.set(20, this.bg.height - 10 - idLbl.height);
                    this.addChild(idLbl);
                }
                let verLbl = this.game.make.text(0, 0, `${scene.platform.appVersion} ${scene.platform.patchVersion}`, fontUtil_1.FontUtil.getTextFont(20, '#ffffff', "normal"));
                verLbl.position.set(this.bg.width - 20 - verLbl.width, this.bg.height - 10 - verLbl.height);
                this.addChild(verLbl);
                this.addDisposable(scene.platform.on("watched_video", this, () => {
                    this.getDailyReward(true);
                }));
            });
        }
        openLinkAccount() {
            const scene = this.getCurrentScene();
            const msgbox = new textMsgBox_1.TextMsgBox(scene, i18n_1.I18n.t("menu.link.account"));
            msgbox.showConfirmButton = true;
            msgbox.on("ok", this, this.startLinkAccountAsync);
            scene.popupWindow(msgbox);
        }
        openUnlinkAccount() {
            const scene = this.getCurrentScene();
            const msgbox = new textMsgBox_1.TextMsgBox(scene, i18n_1.I18n.t("menu.unlink.account"));
            msgbox.showConfirmButton = true;
            msgbox.on("ok", this, this.startUnlinkAccountAsync);
            scene.popupWindow(msgbox);
        }
        startUnlinkAccountAsync() {
            return __awaiter(this, void 0, void 0, function* () {
                this.closeMe();
                const scene = this.getCurrentScene();
                let result = yield scene.platform.logoutGoogleAsync();
                if (result) {
                    this.player.hasLinkAccount = false;
                }
            });
        }
        startLinkAccountAsync() {
            return __awaiter(this, void 0, void 0, function* () {
                this.closeMe();
                const scene = this.getCurrentScene();
                scene.startLoading();
                let result = yield scene.platform.loginGoogleAsync();
                if (!result) {
                    return;
                }
                let gplayerId = yield scene.platform.getGooglePlayerIdAsync();
                if (gplayerId == null) {
                    return;
                }
                const ret = yield scene.platform.linkCustomIdAsync(gplayerId);
                scene.stopLoading();
                const playerLogic = scene.getLogic(playerLogic_1.PlayerLogic);
                if (ret == 0) {
                    yield playerLogic.saveAllDbAsync(true);
                    this.player.hasLinkAccount = true;
                }
                else if (ret == 1) {
                    const msgbox = new textMsgBox_1.TextMsgBox(scene, i18n_1.I18n.t("menu.recover.account"));
                    msgbox.showConfirmButton = true;
                    msgbox.on("ok", this, () => __awaiter(this, void 0, void 0, function* () {
                        scene.startLoading();
                        yield playerLogic.recoverAccountAsync(gplayerId);
                        if (yield scene.platform.loginAsync()) {
                            let newRet = yield scene.platform.linkCustomIdAsync(gplayerId, true);
                            if (newRet == 0) {
                                this.player.hasLinkAccount = true;
                            }
                        }
                        scene.stopLoading();
                        scene.platform.exitApp();
                    }));
                    scene.popupWindow(msgbox);
                }
            });
        }
        openAchievements() {
            const scene = this.getCurrentScene();
            let win = new achievementMenu_1.AchievementMenu(scene);
            scene.popupWindow(win);
        }
        getDailyReward(fromAd = false) {
            const scene = this.getCurrentScene();
            if (this.checkInRemaining < 0 || fromAd) {
                this.checkInRemaining = this.rewardInterval;
                const emitterModel = scene.modelStore.loadByType(this.player.id, "monster_emitter")[0];
                const gameConfig = scene.configUtil.getConfig(gameConfiguration_1.GameConfiguration);
                const difficulty = emitterModel.level;
                const goldReward = (difficulty * difficulty) * 2000;
                const timeReward = gameConfig.getNum("initGauge") * 1000;
                this.player.gold += goldReward;
                this.player.gauge += timeReward;
                this.player.lastGetReward = Date.now();
                logger_1.Logger.p("daily_checkin", { gold: goldReward, difficulty: difficulty, fromAd: fromAd });
                const grp = scene.game.make.group();
                const graphics = scene.game.add.graphics(0, 0, grp);
                graphics.drawRect(0, 0, customMsgBox_1.CustomMsgBox.DefaultWidth, 100);
                const text = scene.game.add.text(graphics.width / 2, 0, i18n_1.I18n.t("You get"), fontUtil_1.FontUtil.getTextFont(36, '#000000', fontUtil_1.FontUtil.DFW, 0), grp);
                text.anchor.set(0.5, 0);
                const goldIcon = scene.charUI.getGoldIcon();
                const timerIcon = scene.charUI.getTimerIcon();
                const goldNumText = scene.game.add.text(0, 0, " + " + goldReward.toString(), fontUtil_1.FontUtil.getNumberFont(28, '#a9fffc'), grp);
                const timerNumText = scene.game.add.text(0, 0, " + " + stringUtil_1.StringUtil.convertFromSeconds(timeReward / 1000), fontUtil_1.FontUtil.getNumberFont(28, '#a9fffc'), grp);
                goldIcon.position.set((graphics.width - goldNumText.width - goldIcon.width) / 2, text.y + text.height);
                timerIcon.position.set((graphics.width - timerNumText.width - timerIcon.width) / 2, goldIcon.y + goldIcon.height);
                goldNumText.position.set(goldIcon.x + goldIcon.width, goldIcon.y);
                timerNumText.position.set(timerIcon.x + timerIcon.width, timerIcon.y);
                grp.add(goldIcon);
                grp.add(timerIcon);
                scene.popupWindow(new customMsgBox_1.CustomMsgBox(scene, grp, graphics.width));
            }
            else {
                let msgbox = new textMsgBox_1.TextMsgBox(scene, i18n_1.I18n.t("menu.adreward"));
                msgbox.showConfirmButton = true;
                this.addDisposable(msgbox.on("ok", this, () => {
                    scene.platform.showVideoAdAsync().then();
                }));
                scene.popupWindow(msgbox);
            }
        }
        openCredit() {
            let credits = ["ぴぽや https://pipoya.net/", "finalbossblues https://finalbossblues.itch.io", "Krachware https://krachware.com/"];
            const scene = this.getCurrentScene();
            const grp = scene.game.make.group();
            let totalWidth = customMsgBox_1.CustomMsgBox.DefaultWidth, totalHeight = 0;
            let startY = 10;
            for (let c of credits) {
                let icon = scene.charUI.getHandIcon();
                icon.position.set(30, startY);
                grp.add(icon);
                let txt = scene.game.add.text(icon.x + icon.width + 10, icon.y, c, fontUtil_1.FontUtil.getTextFont(24, '#f7ff94', "normal"), grp);
                txt.wordWrap = true;
                txt.wordWrapWidth = totalWidth - txt.x - 10;
                totalHeight += txt.height;
                startY += txt.height;
            }
            const graphics = scene.game.add.graphics(0, 0, grp);
            graphics.drawRect(0, 0, totalWidth, totalHeight);
            scene.popupWindow(new customMsgBox_1.CustomMsgBox(scene, grp, graphics.width));
        }
    }
    exports.MiscellaneousMenu = MiscellaneousMenu;
});
//# sourceMappingURL=miscellaneousMenu.js.map