define(["require", "exports", "./baseWindow", "../../utils/i18n", "../../utils/fontUtil", "../characters/characterFactory", "../characters/characterUI", "../../configs/damageCalculationConfiguration", "../../logics/playerLogic"], function (require, exports, baseWindow_1, i18n_1, fontUtil_1, characterFactory_1, characterUI_1, damageCalculationConfiguration_1, playerLogic_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class MonsterDetailWindow extends baseWindow_1.BaseWindow {
        constructor(scene, config, model, record) {
            super(scene);
            this.config = config;
            this.model = model;
            this.record = record;
            this.charUI = new characterUI_1.CharacterUI(this.getCurrentScene());
        }
        preloadRes() {
            super.preloadRes();
            this.charUI.preloadResAsync().then();
            this.checkAndLoadImage(MonsterDetailWindow.ResPrefix + "txtLine", "common/lineC4.png");
            this.checkAndLoadImage(MonsterDetailWindow.ResPrefix + "bg", "common/window/mainwindowE2.png");
        }
        initView() {
            const grp = this.game.add.group(this);
            const scene = this.getCurrentScene();
            const charFact = scene.serviceProvider.getService(characterFactory_1.CharacterFactory);
            let bg = scene.feUI.createPanel(MonsterDetailWindow.ResPrefix + "bg", 55, 560, 640);
            grp.add(bg);
            let npcName = i18n_1.I18n.t(this.config.name);
            let nameLbl = this.game.add.text(0, 0, npcName, fontUtil_1.FontUtil.getTextFont(30, '000000', 'normal', 0), grp);
            nameLbl.position.set(bg.width - nameLbl.width - 35, 15);
            let btn;
            btn = scene.feUI.createRoundGameButtonNine("yellow", i18n_1.I18n.t("menu.share"), this.share, this);
            btn.position.set(bg.width / 2 + btn.width / 4, bg.height - btn.height - 20);
            btn.setLblPos("center");
            grp.add(btn);
            const closeBtn = scene.feUI.createRoundGameButtonNine("red", i18n_1.I18n.t("menu.close"), this.closeMe, this);
            closeBtn.position.set(bg.width / 2 - closeBtn.width / 4 - closeBtn.width, btn.y);
            closeBtn.setLblPos("center");
            grp.add(closeBtn);
            charFact.createMonsterForUI(50, 0, this.config).then((character) => {
                character.walk();
                character.scale.set(128 / character.height);
                grp.addChild(character);
                const startX = 65;
                const startY = character.height + 5;
                this.initDetail(startX, startY, grp);
            });
        }
        addSubLine(txt, parent) {
            let img = this.game.add.image(0, 0, MonsterDetailWindow.ResPrefix + "txtLine", null, parent);
            img.position.set(txt.x - 40, txt.y + txt.height - img.height);
        }
        initDetail(startX, startY, grp) {
            const titleStyle = fontUtil_1.FontUtil.getTextFont(28, '#000000', "bold", 0);
            const fontStyle = fontUtil_1.FontUtil.getTextFont(24, '#000000', "normal", 0);
            let statusBegin = this.game.add.text(startX, startY, i18n_1.I18n.t("menu.stat"), titleStyle, grp);
            this.addSubLine(statusBegin, grp);
            let lvLbl = this.game.add.text(startX + 5, statusBegin.y + statusBegin.height, i18n_1.I18n.t("stat.lv"), fontStyle, grp);
            let spdLbl = this.game.add.text(lvLbl.x, lvLbl.y + lvLbl.height, i18n_1.I18n.t("stat.hp"), fontStyle, grp);
            let criLbl = this.game.add.text(spdLbl.x, spdLbl.y + spdLbl.height, i18n_1.I18n.t("stat.def"), fontStyle, grp);
            let atkLbl = this.game.add.text(criLbl.x, criLbl.y + criLbl.height, i18n_1.I18n.t("stat.defeated"), fontStyle, grp);
            const attrNumFontStyle = fontUtil_1.FontUtil.getTextFont(22, '#000000', 'bold', 0);
            let lv = this.model.level.toString(), atk = this.record.killed.toString(), cri = this.model.defense.toString(), spd = this.model.maxHp.toString();
            let criNumLbl = this.game.add.text(0, 0, cri, attrNumFontStyle, grp);
            criNumLbl.anchor.set(1, 0);
            criNumLbl.position.set(startX + 420, criLbl.y);
            let atkNumLbl = this.game.add.text(0, 0, atk, attrNumFontStyle, grp);
            atkNumLbl.anchor.set(1, 0);
            atkNumLbl.position.set(criNumLbl.x, atkLbl.y);
            let lvNumLbl = this.game.add.text(0, 0, lv, attrNumFontStyle, grp);
            lvNumLbl.anchor.set(1, 0);
            lvNumLbl.position.set(criNumLbl.x, lvLbl.y);
            let spdNumLbl = this.game.add.text(0, 0, spd, attrNumFontStyle, grp);
            spdNumLbl.anchor.set(1, 0);
            spdNumLbl.position.set(criNumLbl.x, spdLbl.y);
            let skillBegin = this.game.add.text(startX, atkLbl.y + atkLbl.height + 10, i18n_1.I18n.t("menu.weakness"), titleStyle, grp);
            this.addSubLine(skillBegin, grp);
            const configUtil = this.getCurrentScene().configUtil;
            const damageCalculationConfig = configUtil.getConfig(damageCalculationConfiguration_1.DamageCalculationConfiguration);
            const weakness = damageCalculationConfig.getRatioByDefType(this.config.defType);
            let skillsStartY = skillBegin.y + skillBegin.height;
            for (let weak of weakness) {
                let icon = this.charUI.getAtkTypeIcon(weak.atkType);
                icon.smoothed = false;
                icon.position.set(lvLbl.x, skillsStartY);
                let ratioNumLbl = this.game.add.text(criNumLbl.x, icon.y, weak.ratio * 100 + "%", attrNumFontStyle, grp);
                ratioNumLbl.anchor.set(1, 0);
                grp.add(icon);
                grp.add(ratioNumLbl);
                skillsStartY += icon.height + 5;
            }
        }
        share() {
            const scene = this.getCurrentScene();
            scene.platform.share(i18n_1.I18n.tn("menu.share.monster", this.record.killed.toString(), i18n_1.I18n.t(this.config.name)), "https://play.google.com/store", () => {
                const logic = scene.getLogic(playerLogic_1.PlayerLogic);
                logic.handleEvent("share", "");
            });
        }
    }
    MonsterDetailWindow.ResPrefix = "mdw_";
    exports.MonsterDetailWindow = MonsterDetailWindow;
});
//# sourceMappingURL=monsterDetailWindow.js.map