var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
define(["require", "exports", "../../baseGameGroup", "../feGameUI", "../../utils/fontUtil", "../listview/listView", "../../utils/i18n", "../characters/characterFactory", "./husbandMenu", "../../configs/monsterConfiguration", "../../models/monsterModel", "../../models/battlefieldModel", "../characters/characterUI", "./monsterDetailWindow"], function (require, exports, baseGameGroup_1, feGameUI_1, fontUtil_1, listView_1, i18n_1, characterFactory_1, husbandMenu_1, monsterConfiguration_1, monsterModel_1, battlefieldModel_1, characterUI_1, monsterDetailWindow_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class MonsterMenuItem extends baseGameGroup_1.BaseGameGroup {
        constructor(scene, monsterConfig) {
            super(scene);
            this._resPrefix = "mmi_";
            this._monsterConfig = monsterConfig;
            this._charUI = new characterUI_1.CharacterUI(this.getCurrentScene());
        }
        preloadResAsync() {
            return __awaiter(this, void 0, void 0, function* () {
                this._charUI.preloadResAsync().then();
                yield this.checkAndLoadImageAsync(this._resPrefix + "bg", "common/window/sabwindowC.png");
            });
        }
        initViewAsync() {
            return __awaiter(this, void 0, void 0, function* () {
                this.removeAll(true);
                const scene = this.getCurrentScene();
                const charFact = scene.serviceProvider.getService(characterFactory_1.CharacterFactory);
                const config = this._monsterConfig;
                const emitter = scene.modelStore.load(battlefieldModel_1.BattlefieldModel.genKey());
                const monsterRecord = emitter.getMonsterRecord(config.id);
                const model = new monsterModel_1.MonsterModel(config.id.toString());
                model.level = monsterRecord.maxLevel;
                model.moveSpeed = config.moveSpeed;
                model.baseMaxHp = config.hp;
                model.defense = config.def;
                let name = this._monsterConfig.name;
                let tex = scene.feUI.createPanel(this._resPrefix + "bg", 40, 560, 120);
                let bgBtn = new feGameUI_1.GameButton(this.getCurrentScene(), tex, null, () => {
                    let win = new monsterDetailWindow_1.MonsterDetailWindow(this.getCurrentScene(), config, model, monsterRecord);
                    scene.popupWindow(win);
                }, this);
                this.add(bgBtn);
                let character = yield charFact.createMonsterForUI(15, tex.height - 15, config);
                character.anchor.set(0, 1);
                character.scale.set(90 / character.height);
                character.walk();
                this.addChild(character);
                let nameLbl = this.game.add.text(120, 10, i18n_1.I18n.t(name), fontUtil_1.FontUtil.getTextFont(30, '#ffffff', 'bold'), this);
                const defTypeIcon = this._charUI.getDefTypeIcon(this._monsterConfig.defType);
                if (defTypeIcon != null) {
                    defTypeIcon.anchor.set(0, 0.5);
                    defTypeIcon.position.set(nameLbl.x + nameLbl.width + 10, nameLbl.y + nameLbl.height / 2);
                    this.add(defTypeIcon);
                }
                const attrFontStyle = fontUtil_1.FontUtil.getTextFont(18, '#ffffff', 'bold');
                const attrNumFontStyle = fontUtil_1.FontUtil.getTextFont(18, '#fcff7f', 'bold');
                let lv = "?", atk = "?", tap = "?";
                let lvLbl = this.game.add.text(nameLbl.x, nameLbl.y + nameLbl.height, i18n_1.I18n.t("stat.lv"), attrFontStyle, this);
                let lvNumLbl = this.game.add.text(lvLbl.x + lvLbl.width + 10, lvLbl.y, lv, attrNumFontStyle, this);
                let killedLbl = this.game.add.text(lvLbl.x + 200, lvLbl.y, i18n_1.I18n.t("stat.defeated"), attrFontStyle, this);
                this.game.add.text(killedLbl.x + killedLbl.width + 10, killedLbl.y, monsterRecord.killed.toString(), attrNumFontStyle, this);
                let hpLbl = this.game.add.text(lvLbl.x, lvLbl.y + lvLbl.height, i18n_1.I18n.t("stat.hp"), attrFontStyle, this);
                let hpNumLbl = this.game.add.text(hpLbl.x + hpLbl.width + 10, hpLbl.y, atk, attrNumFontStyle, this);
                let defLbl = this.game.add.text(hpLbl.x + 200, hpLbl.y, i18n_1.I18n.t("stat.def"), attrFontStyle, this);
                let defNumLbl = this.game.add.text(defLbl.x + defLbl.width + 10, hpLbl.y, tap, attrNumFontStyle, this);
                this.refreshLabels(model, lvNumLbl, hpNumLbl, defNumLbl);
            });
        }
        refreshLabels(model, lvLbl, hpLbl, defLbl) {
            let lv = model.level.toString();
            let atk = model.maxHp.toString();
            let def = model.defense.toString();
            lvLbl.setText(lv);
            hpLbl.setText(atk);
            defLbl.setText(def);
        }
    }
    class MonsterMenu extends husbandMenu_1.HusbandMenu {
        initListViewAsync() {
            return __awaiter(this, void 0, void 0, function* () {
                const scene = this.getCurrentScene();
                const npcConfig = scene.configUtil.getConfig(monsterConfiguration_1.MonsterConfiguration);
                const npcConfigSize = npcConfig.size;
                let listView = null;
                const grp = this.game.make.group();
                const emitter = scene.modelStore.load(battlefieldModel_1.BattlefieldModel.genKey());
                let shown = 0;
                for (let i = 1; i <= npcConfigSize; i++) {
                    let entry = npcConfig.get(i);
                    if (emitter.getMonsterRecord(entry.id) == null) {
                        continue;
                    }
                    shown++;
                    let item = new MonsterMenuItem(this.getCurrentScene(), entry);
                    yield item.preloadResAsync();
                    if (listView == null) {
                        yield item.initViewAsync();
                        const bounds = new Phaser.Rectangle(0, 0, item.width, this.bg.height - this._grayLayerHeight - 40);
                        const options = {
                            direction: 'y',
                            overflow: 100,
                            padding: 10,
                            searchForClicks: true
                        };
                        grp.position.set((this.bg.width - item.width) / 2, this.grayLayer.y + this.grayLayer.height + 5);
                        listView = new listView_1.ListView(this.game, grp, bounds, options);
                    }
                    else {
                        item.initViewAsync().then();
                    }
                    listView.add(item);
                }
                this.add(grp);
                let counterLbl = this.game.make.text(grp.x, this.grayLayer.y + this.grayLayer.height / 2, shown + "/" + npcConfigSize, fontUtil_1.FontUtil.getTextFont(24, '#ffffff'));
                counterLbl.anchor.set(0, 0.5);
                this.addChild(counterLbl);
            });
        }
    }
    exports.MonsterMenu = MonsterMenu;
});
//# sourceMappingURL=monsterMenu.js.map