define(["require", "exports", "./baseWindow", "../feGameUI", "../../utils/i18n", "../../utils/fontUtil", "../../logics/playerLogic", "../characters/characterFactory", "../characters/characterUI", "../../configs/npcDefaultSkillConfiguration", "../../configs/skillConfiguration", "./textMsgBox", "../../models/skillModel"], function (require, exports, baseWindow_1, feGameUI_1, i18n_1, fontUtil_1, playerLogic_1, characterFactory_1, characterUI_1, npcDefaultSkillConfiguration_1, skillConfiguration_1, textMsgBox_1, skillModel_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class NpcDetailWindow extends baseWindow_1.BaseWindow {
        constructor(scene, playerModel, npcConfig, npcModel) {
            super(scene);
            this.playerModel = playerModel;
            this.npcConfig = npcConfig;
            this.npcModel = npcModel;
            this._charUI = new characterUI_1.CharacterUI(this.getCurrentScene());
        }
        preloadRes() {
            super.preloadRes();
            this._charUI.preloadResAsync().then();
            this.checkAndLoadImage(NpcDetailWindow.ResPrefix + "txtLine", "common/lineC4.png");
            this.checkAndLoadImage(NpcDetailWindow.ResPrefix + "bg", "common/window/mainwindowE2.png");
        }
        initView() {
            const grp = this.game.add.group(this);
            const scene = this.getCurrentScene();
            const charFact = scene.serviceProvider.getService(characterFactory_1.CharacterFactory);
            let bg = scene.feUI.createPanel(NpcDetailWindow.ResPrefix + "bg", 55, 560, 640);
            grp.add(bg);
            let npcName = i18n_1.I18n.t(this.npcConfig.name);
            let nameLbl = this.game.add.text(0, 0, npcName, fontUtil_1.FontUtil.getTextFont(40, '000000', 'normal', 0), grp);
            nameLbl.position.set(bg.width - nameLbl.width - 35, 15);
            let btn;
            if (this.npcModel == null) {
                btn = scene.feUI.createRoundGameButtonNine("green", i18n_1.I18n.t("menu.hire"), this.hire, this);
            }
            else {
                btn = scene.feUI.createRoundGameButtonNine("yellow", i18n_1.I18n.t("menu.upgrade"), this.upgrade, this);
            }
            btn.position.set(bg.width / 2 + btn.width / 4, bg.height - btn.height - 20);
            btn.setLblPos("center");
            grp.add(btn);
            const closeBtn = scene.feUI.createRoundGameButtonNine("red", i18n_1.I18n.t("menu.close"), this.closeMe, this);
            closeBtn.position.set(bg.width / 2 - closeBtn.width / 4 - closeBtn.width, btn.y);
            closeBtn.setLblPos("center");
            grp.add(closeBtn);
            charFact.createNpcForUI(0, 0, this.npcConfig).then((character) => {
                character.standby();
                grp.addChild(character);
                const startX = 65;
                const startY = character.height + 5;
                this.initDetail(startX, startY, grp);
            });
        }
        upgrade() {
            const logic = this.getCurrentScene().getLogic(playerLogic_1.PlayerLogic);
            if (this.npcModel != null) {
                logic.upgradeNpc(this.npcModel.genKey());
            }
        }
        hire() {
            const logic = this.getCurrentScene().getLogic(playerLogic_1.PlayerLogic);
            if (this.npcModel == null) {
                logic.buyNpc(this.playerModel.id, this.npcConfig.id);
                this.closeMe();
            }
        }
        addSubLine(txt, parent) {
            let img = this.game.add.image(0, 0, NpcDetailWindow.ResPrefix + "txtLine", null, parent);
            img.position.set(txt.x - 40, txt.y + txt.height - img.height);
        }
        initDetail(startX, startY, grp) {
            const titleStyle = fontUtil_1.FontUtil.getTextFont(28, '#000000', "bold", 0);
            const fontStyle = fontUtil_1.FontUtil.getTextFont(24, '#000000', "normal", 0);
            let statusBegin = this.game.add.text(startX, startY, i18n_1.I18n.t("menu.stat"), titleStyle, grp);
            this.addSubLine(statusBegin, grp);
            let lvLbl = this.game.add.text(startX + 5, statusBegin.y + statusBegin.height, i18n_1.I18n.t("stat.lv"), fontStyle, grp);
            let atkLbl = this.game.add.text(lvLbl.x, lvLbl.y + lvLbl.height, i18n_1.I18n.t("stat.atk"), fontStyle, grp);
            let spdLbl = this.game.add.text(atkLbl.x, atkLbl.y + atkLbl.height, i18n_1.I18n.t("stat.spd"), fontStyle, grp);
            let criLbl = this.game.add.text(spdLbl.x, spdLbl.y + spdLbl.height, i18n_1.I18n.t("stat.cri"), fontStyle, grp);
            let expLbl = this._charUI.getGoldIcon();
            expLbl.position.set(criLbl.x, criLbl.y + criLbl.height);
            grp.add(expLbl);
            const attrNumFontStyle = fontUtil_1.FontUtil.getNumberFont(22, '#000000', 'bold', 0);
            let lv = "?", atk = "?", cri = "?", exp = this.npcConfig.initGold.toString(), spd = "?";
            let criNumLbl = this.game.add.text(0, 0, cri, attrNumFontStyle, grp);
            criNumLbl.anchor.set(1, 0);
            criNumLbl.position.set(startX + 420, criLbl.y);
            let atkNumLbl = this.game.add.text(0, 0, atk, attrNumFontStyle, grp);
            atkNumLbl.anchor.set(1, 0);
            atkNumLbl.position.set(criNumLbl.x, atkLbl.y);
            let lvNumLbl = this.game.add.text(0, 0, lv, attrNumFontStyle, grp);
            lvNumLbl.anchor.set(1, 0);
            lvNumLbl.position.set(criNumLbl.x, lvLbl.y);
            let expNumLbl = this.game.add.text(0, 0, exp, attrNumFontStyle, grp);
            expNumLbl.anchor.set(1, 0);
            expNumLbl.position.set(criNumLbl.x, expLbl.y);
            let spdNumLbl = this.game.add.text(0, 0, spd, attrNumFontStyle, grp);
            spdNumLbl.anchor.set(1, 0);
            spdNumLbl.position.set(criNumLbl.x, spdLbl.y);
            let skillBegin = this.game.add.text(startX, expLbl.y + expLbl.height + 10, i18n_1.I18n.t("menu.skills"), titleStyle, grp);
            this.addSubLine(skillBegin, grp);
            const configUtil = this.getCurrentScene().configUtil;
            const defaultSkillsConfig = configUtil.getConfig(npcDefaultSkillConfiguration_1.NpcDefaultSkillConfiguration);
            const defaultSkills = defaultSkillsConfig.get(this.npcConfig.id);
            const skillsConfig = configUtil.getConfig(skillConfiguration_1.SkillConfiguration);
            let skillsStartY = skillBegin.y + skillBegin.height;
            let skillIdSet = new Set();
            for (let ds of defaultSkills) {
                if (skillIdSet.has(ds.skillId)) {
                    continue;
                }
                skillIdSet.add(ds.skillId);
                const skillConfig = skillsConfig.getConfig(ds.skillId);
                let suffix = "";
                if (skillModel_1.SkillModel.isRushOnly(ds.conditionType)) {
                    suffix = "(" + i18n_1.I18n.t("RUSH") + ")";
                }
                let skillLbl = this.game.add.text(lvLbl.x, skillsStartY, i18n_1.I18n.t(skillConfig.name) + suffix, fontStyle, grp);
                let question = this._charUI.getQuestionIcon();
                question.anchor.set(1, 0);
                let questionBtn = new feGameUI_1.GameButton(this.getCurrentScene(), question, null, () => {
                    let msgbox = new textMsgBox_1.TextMsgBox(this.getCurrentScene(), i18n_1.I18n.t(skillConfig.desc));
                    this.getCurrentScene().popupWindow(msgbox);
                }, this);
                questionBtn.position.set(criNumLbl.x, skillsStartY);
                grp.add(questionBtn);
                skillsStartY += skillLbl.height + 10;
            }
            if (this.npcModel != null) {
                this.refreshLabels(this.npcModel, lvNumLbl, atkNumLbl, criNumLbl, expNumLbl, spdNumLbl);
                this.addDisposable(this.npcModel.onPropertyChangedS("level", this, () => {
                    this.refreshLabels(this.npcModel, lvNumLbl, atkNumLbl, criNumLbl, expNumLbl, spdNumLbl);
                }));
            }
        }
        refreshLabels(model, lvLbl, atkLbl, criLbl, expLbl, spdLbl) {
            let lv = model.level.toString();
            let atk = model.minAttack.toString() + " - " + model.maxAttack.toString();
            let tap = model.criticalRate + "%";
            let exp = model.exp.toString();
            let spd = (1000 / model.speed).toFixed(1);
            lvLbl.setText(lv);
            atkLbl.setText(atk);
            criLbl.setText(tap);
            expLbl.setText(exp);
            spdLbl.setText(spd);
        }
    }
    NpcDetailWindow.ResPrefix = "ndw_";
    exports.NpcDetailWindow = NpcDetailWindow;
});
//# sourceMappingURL=npcDetailWindow.js.map