define(["require", "exports", "./baseWindow", "../../utils/fontUtil", "../../baseGameGroup", "../simpleRadioGroup"], function (require, exports, baseWindow_1, fontUtil_1, baseGameGroup_1, simpleRadioGroup_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class TabItem extends baseGameGroup_1.BaseGameGroup {
        constructor(scene, name, callback, callbackContext) {
            super(scene);
            this._status = 0;
            const partScale = 0.5;
            this._btn = scene.game.make.button(0, 0, "win_tab2", this.onPressed, this, null, null);
            this._callback = callback;
            this._callbackContext = callbackContext;
            this.add(this._btn);
            this._titleLabel = this.game.make.text(this._btn.width / 2, this._btn.height / 2, name, fontUtil_1.FontUtil.getTabTitleFont(false));
            this._titleLabel.anchor.set(0.5, 0.6);
            this._titleLabel.scale.set(1 / partScale);
            this.scale.set(partScale);
            this.addChild(this._titleLabel);
            this.deactivated();
        }
        get status() {
            return this._status;
        }
        set radioGroup(value) {
            this._radioGroup = value;
        }
        activated() {
            this._status = 1;
            this._btn.loadTexture("win_tab1");
            this._titleLabel.setStyle(fontUtil_1.FontUtil.getTabTitleFont(true));
            if (this.parent instanceof Phaser.Group) {
                this.parent.sort("status");
            }
        }
        deactivated() {
            this._status = 0;
            this._btn.loadTexture("win_tab2");
            this._titleLabel.setStyle(fontUtil_1.FontUtil.getTabTitleFont(false));
        }
        enable() {
            this.onPressed();
        }
        onPressed() {
            if (this._radioGroup) {
                this._radioGroup.onItemSelected(this);
            }
            if (this._callback) {
                this._callback.apply(this._callbackContext ? this._callbackContext : this, [this]);
            }
        }
    }
    class TabbedDefaultWindow extends baseWindow_1.DefaultWindow {
        constructor(scene, tabNames) {
            super(scene);
            this._pages = [];
            this._tabs = [];
            this._tabNames = [];
            this._currentTab = 0;
            this._tabNames = tabNames;
        }
        set currentTab(value) {
            this._currentTab = value;
        }
        initView() {
            super.initView();
            const radioGroup = new simpleRadioGroup_1.RadioGroup();
            let tab1 = new TabItem(this.getCurrentScene(), this._tabNames[0], this.onTabPressed, this);
            tab1.position.set(10, this.leftBottom.y + this.leftBottom.height);
            tab1.radioGroup = radioGroup;
            this.add(tab1);
            radioGroup.addItem(tab1);
            this._tabs.push(tab1);
            let tab2 = new TabItem(this.getCurrentScene(), this._tabNames[1], this.onTabPressed, this);
            tab2.position.set(this.winWidth / 2 - 10, tab1.y);
            this.add(tab2);
            tab2.radioGroup = radioGroup;
            radioGroup.addItem(tab2);
            this._tabs.push(tab2);
        }
        onShow() {
            super.onShow();
            this._tabs[this._currentTab].enable();
        }
        onTabPressed(sender) {
            for (let i = 0; i < this._tabs.length; i++) {
                let tab = this._tabs[i];
                if (tab == sender) {
                    let page = this._pages[i];
                    if (page != null && this._currentPage != page) {
                        if (this._currentPage != null) {
                            this._currentPage.visible = false;
                        }
                        page.position.set(this.leftUpper.x + this.leftUpper.width, this.leftUpper.y + this.leftUpper.height);
                        page.pageWidth = this.rightUpper.x - page.position.x;
                        page.pageHeight = this.leftBottom.y + this.leftBottom.height / 2 - page.y;
                        if (page.parent == null) {
                            this.add(page);
                        }
                        else {
                            page.visible = true;
                        }
                        this._currentPage = page;
                        page.onShow();
                    }
                    break;
                }
            }
        }
        addPage(page) {
            this._pages.push(page);
        }
    }
    exports.TabbedDefaultWindow = TabbedDefaultWindow;
});
//# sourceMappingURL=tabbedDefaultWindow.js.map