var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
define(["require", "exports", "./platformUtil", "../configs/gameConfiguration"], function (require, exports, platformUtil_1, gameConfiguration_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class AndroidUtil extends platformUtil_1.PlatformUtil {
        get isMobile() {
            return true;
        }
        set playFabId(value) {
            super.playFabId = value;
            if (value != null && value.length > 0) {
                AppCenter.setUserId(value, () => {
                }, () => {
                });
                this.sentry.configureScope((scope) => {
                    scope.setUser({ "id": value });
                });
            }
        }
        get playFabId() {
            return super.playFabId;
        }
        get appVersion() {
            return this._appVersion;
        }
        get patchVersion() {
            return this._patchVersion;
        }
        get storeUrl() {
            if (this._storeUrl == null) {
                return "";
            }
            return this._storeUrl;
        }
        initAsync() {
            const _super = Object.create(null, {
                initAsync: { get: () => super.initAsync }
            });
            return __awaiter(this, void 0, void 0, function* () {
                yield _super.initAsync.call(this);
                this._appVersion = yield cordova.getAppVersion.getVersionNumber();
                this.sentry = cordova.require("sentry-cordova.Sentry");
                this.sentry.init({ dsn: 'https://f675f55ab49e4eef81bd3a435bfb6ef5@sentry.io/1422812', release: this._appVersion });
                const gc = this.configUtil.getConfig(gameConfiguration_1.GameConfiguration);
                this.gameConfig = gc;
                const codePush = cordova.require("cordova-plugin-code-push.codePush");
                codePush.sync();
                codePush.getCurrentPackage((localPack) => {
                    if (localPack == null) {
                        this._patchVersion = "v0";
                    }
                    else {
                        this._patchVersion = localPack.label;
                    }
                }, null);
                document.addEventListener("backbutton", () => {
                    this.emit("back_btn", this);
                }, false);
                document.addEventListener('admob.rewardvideo.events.LOAD', () => {
                    this.emit("video_ready", this);
                }, false);
                document.addEventListener('admob.rewardvideo.events.REWARD', () => {
                    this.emit("watched_video", this);
                }, false);
                document.addEventListener('admob.rewardvideo.events.CLOSE', () => {
                    this.prepareVideoAdAsync().then();
                }, false);
                this.admob = cordova.require("cordova-plugin-admob-free.AdMob");
                this._storeUrl = gc.getStr("googlePlayUrl");
            });
        }
        getUUIDAsync() {
            return __awaiter(this, void 0, void 0, function* () {
                return device.uuid;
            });
        }
        loginAsync() {
            return __awaiter(this, void 0, void 0, function* () {
                const uuid = yield this.getUUIDAsync();
                let request = {
                    AndroidDeviceId: uuid,
                    CreateAccount: true,
                    TitleId: this.playFabTitleId,
                    AndroidDevice: device.model,
                    OS: device.version
                };
                console.log(`deviceModel [${device.model}] deviceOS [${device.version}]`);
                return new Promise((resolve) => {
                    PlayFabClientSDK.LoginWithAndroidDeviceID(request, this.loginPFCallback(resolve));
                });
            });
        }
        showAdAsync() {
            return __awaiter(this, void 0, void 0, function* () {
                if (this.banner == null) {
                    this.banner = this.admob.banner;
                    this.banner.config({ id: this.gameConfig.getStr("bannerAd") });
                    yield this.banner.prepare();
                }
                yield this.banner.show();
                yield this.prepareVideoAdAsync();
            });
        }
        hideAdAsync() {
            return __awaiter(this, void 0, void 0, function* () {
                if (this.banner == null) {
                    return;
                }
                yield this.banner.remove();
                this.banner = null;
            });
        }
        prepareVideoAdAsync() {
            return __awaiter(this, void 0, void 0, function* () {
                if (this.video == null) {
                    this.video = this.admob.rewardvideo;
                }
                else {
                    const ready = yield this.video.isReady();
                    if (ready) {
                        return;
                    }
                }
                this.video.config({ id: this.gameConfig.getStr("videoAd") });
                yield this.video.prepare();
            });
        }
        showVideoAdAsync() {
            return __awaiter(this, void 0, void 0, function* () {
                if (this.video == null) {
                    return;
                }
                const ready = yield this.video.isReady();
                if (ready) {
                    yield this.video.show();
                }
            });
        }
        exitApp() {
            super.exitApp();
            let nav = navigator;
            if (nav.app) {
                nav.app.exitApp();
            }
            else if (nav.device) {
                nav.device.exitApp();
            }
            else {
                window.close();
            }
        }
        share(message, url, onSuccess, onFail) {
            const options = {
                message: message,
                url: url,
            };
            window.plugins.socialsharing.shareWithOptions(options, onSuccess, onFail);
        }
        hasLoggedInGoogleAsync() {
            return __awaiter(this, void 0, void 0, function* () {
                return new Promise((resolve) => {
                    window.plugins.playGamesServices.isSignedIn((result) => {
                        if (result.isSignedIn) {
                            resolve(true);
                        }
                        else {
                            resolve(false);
                        }
                    }, () => {
                        resolve(false);
                    });
                });
            });
        }
        getGooglePlayerIdAsync() {
            return __awaiter(this, void 0, void 0, function* () {
                return new Promise((resolve) => {
                    window.plugins.playGamesServices.showPlayer((result) => {
                        resolve(result.playerId);
                    }, () => {
                        resolve(null);
                    });
                });
            });
        }
        loginGoogleAsync() {
            return __awaiter(this, void 0, void 0, function* () {
                return new Promise((resolve) => {
                    window.plugins.playGamesServices.auth(() => {
                        resolve(true);
                    }, () => {
                        resolve(false);
                    });
                });
            });
        }
        logoutGoogleAsync() {
            return __awaiter(this, void 0, void 0, function* () {
                return new Promise((resolve) => {
                    window.plugins.playGamesServices.signOut(() => {
                        resolve(true);
                    }, () => {
                        resolve(false);
                    });
                });
            });
        }
        updateAchievementsAsync(achievementServerId) {
            return __awaiter(this, void 0, void 0, function* () {
                let r = yield this.hasLoggedInGoogleAsync();
                if (r) {
                    const data = {
                        achievementId: achievementServerId
                    };
                    window.plugins.playGamesServices.unlockAchievement(data);
                }
            });
        }
    }
    exports.AndroidUtil = AndroidUtil;
});
//# sourceMappingURL=androidUtil.js.map