define(["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class ConfigUtil {
        constructor() {
            this._map = new Map();
        }
        set game(value) {
            this._game = value;
            this._game.load.onFileComplete.add(this.fileCompleted, this);
        }
        addConfig(wrapper) {
            this._map.set(wrapper.name, wrapper);
            this._game.load.json(wrapper.name, "configs/" + wrapper.configName + ".json");
        }
        addEncryptedConfig(wrapper) {
            this._map.set(wrapper.name, wrapper);
            this._game.load.text(wrapper.name, "configs/" + wrapper.configName + ".bin");
        }
        getConfig(c) {
            return this._map.get(c.name);
        }
        fileCompleted(progress, cacheKey, success, totalLoaded, totalFiles) {
            if (!this._map.has(cacheKey)) {
                return;
            }
            if (success) {
                let temp;
                if (this._game.cache.checkJSONKey(cacheKey)) {
                    temp = this._game.cache.getJSON(cacheKey);
                }
                else {
                    const str = this._game.cache.getText(cacheKey);
                    const len = str.length;
                    const int_to_minus = len % 31;
                    let magic_num = 1;
                    let origin = '';
                    for (let i = 0; i < len; i++) {
                        origin = origin + String.fromCharCode(str.charCodeAt(i) - int_to_minus - magic_num);
                        magic_num += 1;
                        if (magic_num > 4) {
                            magic_num = 1;
                        }
                    }
                    temp = JSON.parse(atob(origin));
                }
                this._map.get(cacheKey).initWithJson(temp);
            }
            else {
                console.warn("load config [" + cacheKey + "] failed");
            }
        }
    }
    exports.ConfigUtil = ConfigUtil;
});
//# sourceMappingURL=configUtil.js.map