define(["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class EngineUtil {
        static checkAndLoadImage(game, key, path, callback, callbackContext) {
            if (game.cache.checkImageKey(key)) {
                if (callback != null) {
                    setTimeout(() => {
                        callback.call(callbackContext);
                    }, 0);
                }
            }
            else {
                game.load.image(key, path);
                if (callback != null) {
                    game.load.onLoadComplete.addOnce(callback, callbackContext);
                    game.load.start();
                }
            }
        }
        static checkAndLoadSpriteSheet(game, key, path, frameWidth, frameHeight, margin, spacing, callback, callbackContext) {
            if (game.cache.checkImageKey(key)) {
                if (callback != null) {
                    setTimeout(() => {
                        callback.call(callbackContext);
                    }, 0);
                }
            }
            else {
                game.load.spritesheet(key, path, frameWidth, frameHeight, undefined, margin, spacing);
                if (callback != null) {
                    game.load.onLoadComplete.addOnce(callback, callbackContext);
                    game.load.start();
                }
            }
        }
        static loadStart(g) {
            if (g == null) {
                throw new Error("g cannot be null");
            }
            setTimeout(() => {
                g.load.start();
            }, 0);
        }
        static loadStartAsync(g) {
            return new Promise((resolve) => {
                g.load.onLoadComplete.addOnce(resolve);
                EngineUtil.loadStart(g);
            });
        }
        static tweenStartAsync(tween) {
            return new Promise((resolve) => {
                tween.onComplete.addOnce(resolve);
                tween.start();
            });
        }
    }
    exports.EngineUtil = EngineUtil;
});
//# sourceMappingURL=engineUtil.js.map