define(["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class Logger {
        static c(name, characterId, src) {
            Logger.l.c(name, characterId, src);
        }
        static p(name, src) {
            Logger.l.p(name, src);
        }
        static t(name, src) {
            Logger.l.t(name, src);
        }
    }
    exports.Logger = Logger;
    class ConsoleLogger extends Logger {
        constructor() {
            super();
            Logger.l = this;
        }
        c(name, characterId, src) {
            console.log(name, characterId, src);
        }
        p(name, src) {
            console.log(name, src);
        }
        t(name, src) {
            console.log(name, src);
        }
    }
    exports.ConsoleLogger = ConsoleLogger;
    class PlayFabLogger extends ConsoleLogger {
        c(name, characterId, src) {
            if (PlayFabClientSDK.IsClientLoggedIn()) {
                const event = {
                    CharacterId: characterId,
                    EventName: name,
                    Body: src
                };
                PlayFabClientSDK.WriteCharacterEvent(event, null);
            }
            else {
                super.c(name, characterId, src);
            }
        }
        p(name, src) {
            if (PlayFabClientSDK.IsClientLoggedIn()) {
                const event = {
                    EventName: name,
                    Body: src
                };
                PlayFabClientSDK.WritePlayerEvent(event, null);
            }
            else {
                super.p(name, src);
            }
        }
        t(name, src) {
            if (PlayFabClientSDK.IsClientLoggedIn()) {
                const event = {
                    EventName: name,
                    Body: src
                };
                PlayFabClientSDK.WriteTitleEvent(event, null);
            }
            else {
                super.t(name, src);
            }
        }
    }
    exports.PlayFabLogger = PlayFabLogger;
});
//# sourceMappingURL=logger.js.map