var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
define(["require", "exports", "../configs/gameConfiguration", "./eventbus"], function (require, exports, gameConfiguration_1, eventbus_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class PlatformUtil {
        constructor() {
            this.eventBus = new eventbus_1.Eventbus();
        }
        set configUtil(value) {
            this._configUtil = value;
        }
        get configUtil() {
            return this._configUtil;
        }
        set playFabId(value) {
            this._playFabId = value;
        }
        get playFabId() {
            return this._playFabId;
        }
        get serviceProvider() {
            return this._serviceProvider;
        }
        set serviceProvider(value) {
            this._serviceProvider = value;
        }
        get playFabTitleId() {
            if (this._playFabTitleId == null) {
                return "D18E";
            }
            return this._playFabTitleId;
        }
        get language() {
            return navigator.language;
        }
        get appVersion() {
            return "0.0.0";
        }
        get patchVersion() {
            return "v0";
        }
        get storeUrl() {
            return "";
        }
        loginPFCallback(resolve) {
            return (result, error) => {
                if (result != null) {
                    this.playFabId = result.data.PlayFabId;
                    console.log(`login [${this._playFabId}]`);
                    resolve(true);
                }
                else {
                    if (error != null) {
                        console.error(error);
                    }
                    resolve(false);
                }
            };
        }
        initAsync() {
            return __awaiter(this, void 0, void 0, function* () {
                const gc = this._configUtil.getConfig(gameConfiguration_1.GameConfiguration);
                this._playFabTitleId = gc.getStr("playFabTitleId");
                PlayFab.settings.titleId = this._playFabTitleId;
            });
        }
        showAdAsync() {
            return __awaiter(this, void 0, void 0, function* () {
            });
        }
        hideAdAsync() {
            return __awaiter(this, void 0, void 0, function* () {
            });
        }
        prepareVideoAdAsync() {
            return __awaiter(this, void 0, void 0, function* () {
            });
        }
        showVideoAdAsync() {
            return __awaiter(this, void 0, void 0, function* () {
            });
        }
        exitApp() {
        }
        share(message, url, onSuccess, onFail) {
        }
        on(event, caller, listener) {
            return this.eventBus.on(event, (event) => {
                listener.call(caller, event);
            });
        }
        emit(event, data) {
            this.eventBus.emit(event, data);
        }
        linkCustomIdAsync(customId, force = false) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`linking customId to playfab`);
                if (!PlayFabClientSDK.IsClientLoggedIn()) {
                    return -1;
                }
                return new Promise((resolve) => {
                    const request = {
                        CustomId: customId,
                        ForceLink: force
                    };
                    PlayFabClientSDK.LinkCustomID(request, (result, error) => {
                        if (error) {
                            if (error.errorCode == 1184) {
                                resolve(1);
                            }
                            else {
                                console.error(error);
                                resolve(-1);
                            }
                        }
                        else {
                            resolve(0);
                        }
                    });
                });
            });
        }
        loginAndFetchUserDataAsync(newCustomId) {
            return __awaiter(this, void 0, void 0, function* () {
                return new Promise((resolve) => {
                    let request = {
                        CustomId: newCustomId,
                        CreateAccount: false,
                        TitleId: this.playFabTitleId,
                        InfoRequestParameters: {
                            GetUserData: true,
                            GetCharacterInventories: false,
                            GetCharacterList: false,
                            GetPlayerProfile: false,
                            GetPlayerStatistics: false,
                            GetTitleData: false,
                            GetUserAccountInfo: false,
                            GetUserInventory: false,
                            GetUserReadOnlyData: false,
                            GetUserVirtualCurrency: false
                        }
                    };
                    PlayFabClientSDK.LoginWithCustomID(request, (result, error) => {
                        if (result == null || result.data == null || result.data.InfoResultPayload == null) {
                            resolve(null);
                        }
                        else if (error != null) {
                            resolve(null);
                            console.error(error);
                        }
                        else {
                            resolve(result.data.InfoResultPayload.UserData);
                        }
                    });
                });
            });
        }
        updateAchievementsAsync(achievementServerId) {
            return __awaiter(this, void 0, void 0, function* () {
            });
        }
        updatePlayerDataAsync(obj) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`updating player data on playfab`);
                if (!PlayFabClientSDK.IsClientLoggedIn()) {
                    return;
                }
                const req = {
                    Data: obj
                };
                PlayFabClientSDK.UpdateUserData(req, null);
            });
        }
    }
    exports.PlatformUtil = PlatformUtil;
});
//# sourceMappingURL=platformUtil.js.map